package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.*;
import java.util.*;
import java.awt.*;  
import javax.swing.*;
/* this class has the utilities to implement any filter developed using the tic-tac-toe program. 
It uses an array of 256 integers which which encode what to do to the central pixel in a 3x3 square
dependeing on the contents of the surrounding pixels. There are only 256 possible 
surrounding patterns. */

class  EightPtFilter {
	static final int [] pows2 = {128,64,32,16,8,4,2,1};
	int rasColDim=-1;
	int [] neighOffs = new int[8];
	static final short initOff=0,erasedOff=0,filledOn=255,initOn=255;
		
	public int applyEightPointPatternFilter(short [] fromras, short [] toras,
			int ncols, int nrows, int [] doWhat) {
		int row,col,patrn,j,i,pixoff,act,numChanged=0;
		if(rasColDim != ncols) { // rebuild offsets
			talk("new GapFill raster size with "+ncols+" columns and "+nrows+" rows.");
			rasColDim=ncols;
			neighOffs[0]=-ncols-1; neighOffs[1]=-ncols; neighOffs[2]=-ncols+1;
			neighOffs[3]=-1;                         neighOffs[4]=1;
			neighOffs[5]=ncols-1;  neighOffs[6]=ncols;  neighOffs[7]=ncols+1;
			}

		for(row=1;row<nrows-1;row++) // omit boundaries
			for(col=1;col<ncols-1;col++) {
				pixoff = row*ncols+col;
				patrn = 0;
				for(j=0;j<8;j++)
					if(fromras[pixoff+neighOffs[j]]>127)
						patrn = patrn | pows2[j];
                if((act = doWhat[patrn]) == 2 || act==0) 
                     //                    noChange     undecided
					toras[pixoff] = fromras[pixoff];
				else if(act==1) {// erase it
					if(fromras[pixoff] > 127) {
						toras[pixoff] = erasedOff;
						++numChanged;
						}
					else
						toras[pixoff] = fromras[pixoff];
					}
				else if (act==3) { // FILL IT
					if(fromras[pixoff] > 127) 
						toras[pixoff] = fromras[pixoff];	
					else {
						toras[pixoff] = filledOn;	
						++numChanged;
						}
					}
				}
		return numChanged;				
		}

	public boolean [] getStatesFromInt(int ipt, boolean isCenterCellOn) {
		int i,k=128;
		boolean [] onoff = new boolean[9];
		for(i=0;i<9;i++)
			if(i == 4)
				onoff[i] = isCenterCellOn;
			else {
				onoff[i] = (ipt & k) != 0;
				k = k/2;
				}
		return onoff;
		}

	public void typePats(int [] thePats) {
		int fillpatrow=0,fillpatcol=0,erasepatrow=0,erasepatcol=0;
		int [][] fillpats = new int[12][25];
		int [][] erasepats = new int[12][25];
		int i,j,index;
		for(index=0;index<256;index++) { // collect together the erase and fill patterns
			i=thePats[index];
			if(i==1) { // it's an erase pattern
				erasepats[erasepatrow][erasepatcol] = index;
				++erasepatcol;
				if(erasepatcol>24) {
					erasepatcol=0;
					++erasepatrow;
					}
				}
			if(i==3) { // it's a fill pattern
				fillpats[fillpatrow][fillpatcol] = index;
				++fillpatcol;
				if(fillpatcol>24) {
					fillpatcol=0;
					++fillpatrow;
					}
				}
			}
		
		if(fillpatrow>0 || fillpatcol>0) {
			talk("Patterns for which central pixel is filled...");
			for(i=0;i<=fillpatrow;i++) {
				j = i==fillpatrow? fillpatcol : 25;
				drawpats(fillpats[i],j,"#");
				}
			}
			
		if(erasepatrow>0 || erasepatcol>0) {
			talk("Patterns for which central pixel is erased...");
			for(i=0;i<=erasepatrow;i++) {
				j = i==erasepatrow? erasepatcol : 25;
				drawpats(erasepats[i],j,"-");
				}
			}	
		}
		
	private void drawpats(int pats[],int ncols,String centr) {
		String top="",mid="",bot="";
		int i,j,index;
		boolean [] states; /*  0  1   2
		                       3  4   5
		                       6  7   8 */
		for(i=0;i<ncols;i++) {
			states = getStatesFromInt(pats[i], false);
			top += states[0]? "*" : "'";
			top += states[1]? "*" : "'";
			top += ((states[2]? "*" : "'")+"  ");
			
			
			mid += states[3]? "*" : "'";
			mid += centr;
			mid += ((states[5]? "*" : "'")+"  ");

			bot += states[6]? "*" : "'";
			bot += states[7]? "*" : "'";
			bot += ((states[8]? "*" : "'")+"  ");
			}
		talk(top) ; talk(mid);  talk(bot); talk("  ");		
		}
		
	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
		
}
