package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.*;
import java.util.*;
import java.awt.*;  
import javax.swing.*;

class  FillThread {
	SegmenterNode nd=null;
	short [] workras = null;
	int workpix=-1; // number of pixels in workras array
	EightPtFilter epf=null;
	static final short initOff=0,erasedOff=0,filledOn=255,initOn=255;
//       0 or 2 => no change to center pixel, 1 => erase center pixel,
//            3 => fill central pixel
static final int [] changes =  { 
2, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 3, 0, 2, 1, 0, 3, 1, 2, 0, 0, 1, 1, 3, 3, 2, 2, 3, 0, 2, 2, 
0, 0, 3, 0, 3, 0, 0, 0, 0, 1, 2, 1, 0, 0, 1, 2, 3, 3, 0, 0, 0, 0, 0, 0, 3, 0, 2, 0, 3, 0, 0, 1, 
0, 3, 3, 3, 3, 3, 3, 0, 1, 2, 2, 2, 0, 0, 2, 0, 1, 0, 2, 2, 2, 0, 2, 0, 2, 2, 2, 3, 2, 3, 3, 3, 
0, 0, 3, 0, 0, 0, 3, 0, 1, 1, 2, 2, 0, 0, 0, 1, 2, 0, 2, 3, 1, 0, 0, 0, 2, 0, 3, 3, 0, 0, 3, 3, 
0, 3, 3, 0, 0, 0, 0, 0, 3, 0, 0, 0, 3, 0, 0, 0, 0, 0, 2, 1, 1, 0, 1, 2, 3, 3, 2, 0, 0, 0, 0, 1, 
0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 
0, 0, 3, 3, 0, 0, 0, 0, 2, 1, 2, 0, 0, 0, 3, 0, 1, 0, 2, 0, 1, 0, 2, 1, 2, 0, 3, 3, 0, 0, 3, 3, 
1, 0, 0, 0, 0, 0, 0, 0, 1, 2, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 2, 0, 1, 0, 2, 1, 3, 3, 1, 0, 3, 2};

	public FillThread(SegmenterNode nd) {
		this.nd=nd;
		epf = new EightPtFilter();
		} 
		
	public void tellWhatYouDo() {
		nd.talk("");
		nd.talk("The FillThread filter has one parameter, iter8, which tells how many");
		nd.talk("successive iterations to perform.");
		nd.talk("It fill/erases pixels which appear to make continuous lines.");
		epf.typePats(changes);
		nd.talk("When you enter iter8== -1, this filter runs in from the edges of the");
		nd.talk("filter and turns on all pixels until it hits an 'on' pixel in its input. The intent is");
		nd.talk("to include in a mask all pixels outside an unbroken rim...");
		}

private boolean rimmer(short [] inras,int NCOLS,int NROWS, int iter8, short[] outras) {
	int row,col,rowoff;
	for(row=0;row<NCOLS*NROWS;row++)
		outras[row]=inras[row];
	// fill edges of the rows
	for(row=0;row<NROWS;row++) {
		//fill from left edge
		rowoff = row*NCOLS;
		col=0;
		if(inras[rowoff+col]<128) 
		  while(col<NCOLS && inras[rowoff+col]<128) {
			outras[rowoff+col] = initOn;
			++col;
			}
		//fill from right edge
		col = NCOLS-1;
		if(inras[rowoff+col]<128) 
		  while(col>=0 && inras[rowoff+col]<128) {
			outras[rowoff+col] = initOn;
			--col;
			}
		}
 // fill tops and botoms of columns
 	for(col=0;col<NCOLS;col++) {
 		row=0;
		if(inras[row*NCOLS+col] <128)
		  while(row < NROWS && inras[row*NCOLS+col] <128) {
			outras[row*NCOLS+col]=initOn;;
			++row;
			}
 		row=NROWS-1;
		if(inras[row*NCOLS+col] <128)
		  while(row >=0 && inras[row*NCOLS+col] <128) {
			outras[row*NCOLS+col]=initOn;;
			--row;
			}
		}
 	return true;			
	}	
		
public boolean doit(short [] inras,int NCOLS,int NROWS, int iter8, short[] outras) {
	int i,numChanges=0, npix=NCOLS*NROWS,offset;
	if(iter8<0) return rimmer(inras, NCOLS, NROWS, iter8, outras);
	if(workpix != npix || workras==null) { // make new work array
		workras = new short[npix];
		workpix=npix;
		}
	// don't change inras...
	for(i=0;i<npix;i++)
		workras[i]=inras[i]>127? initOn : initOff;
	
	if(iter8<0) iter8=256;
	numChanges=10;
	
	while(iter8>0 && numChanges>0) {
		numChanges=epf.applyEightPointPatternFilter(workras, outras,NCOLS,NROWS,changes);
		--iter8;
		if(iter8<=0 || numChanges<=0) return true;
		// do it again, but put answer back into workras
		numChanges=epf.applyEightPointPatternFilter(outras, workras,NCOLS,NROWS,changes);
		--iter8;		
		}
	// if fall through loop, the answer is in the workras array.
	for(i=0;i<npix;i++) 
		outras[i]=workras[i]; // copy workras to the output array	

	return true;
	}

	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
					
}