package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.*;
import java.util.*;
import java.awt.*;  import javax.swing.*;

class FocusFltr {
	SegmenterNode nd=null;
	int [] coloffsets = null;
	int [] rowoffsets = null;
	float [] foci = null;
	int nfoci=0;
	int nrows=-1,ncols=-1;
	int indisk=-1;
	int diskRadius=-1;
	
	private double histogramWidth(int row, int col, short [] inras) {
		int i,r,c,insum=0;
		double brisum=0., brisqsum=0,bri,avg;
		/* n = number of points in histogram
		avg = sum (brightness[i])/n
		stddev = sqrt( sum( (brightness[i] - avg)^2) ) / n
			sqrt(sum(brightness[i]^2) - sum(2*avg*brightness[i]) +  n*avg^2  ))/n  
			= sqrt(sum(brightness[i]^2) - 2*avg*sum(brightness[i]) +  n*avg^2  ))/n */
		for(i=0;i<indisk;i++) 
			if( (r=row+rowoffsets[i]) >=0 && r < nrows && (c=col+ coloffsets[i])>=0 && c <ncols) {
				bri = inras[r*ncols+c];
				brisum += bri;
				brisqsum += (bri*bri);
				++insum;
				}
		avg = brisum/(double)insum;
		bri = (brisqsum - 2.*avg*brisum + insum*avg*avg);
		return (Math.sqrt(bri)/(double)insum);
		}
		
	public FocusFltr(SegmenterNode nd) {
		this.nd=nd;
		}

	public void tellWhatYouDo() {
		nd.talk("");
		nd.talk("The Focus filter has one parameters, (int)rad = radius of the disk surrounding each pixel.");
		nd.talk("The standard deviation of the histogram of brightness values within this disk");
		nd.talk("measures how sharp the focus is at that pixel. The output at each pixel is the value of");
		nd.talk("that std dev, normalized to 255. The intent is that you threshold the output of this filter");
		nd.talk("to make a mask delineating pixels that are in sharpest focus. ");
		}
		

public boolean doit(short [] inras , int incols,   int inrows, int rad,short [] outras) {
		int i, j, voxOn, npix = inrows*incols,numAboveThreshold=0, nsharpest=0;
		int x,y,r,c,radsq = rad*rad;
		int inWidthOGram = 256;
		double sum=0.,avgWidth, maxWidth=-1.,histWidth;
		if(nd.diags) nd.talk("FocusFltr with rad = "+rad);
		if(rad != diskRadius || incols != ncols || inrows != nrows || coloffsets == null) {
			// remake the offsets array
			coloffsets = new int[2+ (2*rad+1)*(2*rad+1)];
			rowoffsets = new int[2+ (2*rad+1)*(2*rad+1)];
			foci = new float[inrows*incols];
			nfoci = inrows*incols;
			indisk=0;
			nrows = inrows;
			ncols = incols;
			diskRadius=rad;
			for(r = -rad;r <= rad; r++)
				for(c =- rad; c<= rad; c++)
					if( r*r + c*c <= radsq ) {
						coloffsets[indisk] = c;
						rowoffsets[indisk] = r; // the diskoffsets[] array ends up with the
						++indisk;// offset from a central point to a pixel inside the disk of radius rad
						}						
			// nd.talk("Remade the disk offsets[] array, indisk = "+indisk);
			}
		
		for(r=0; r<inrows; r++) 
			for(c=0; c<incols; c++)  {
				histWidth = histogramWidth(r,c,inras);
				if(histWidth>maxWidth)
					maxWidth=histWidth;
				foci[r*incols+c] = (float)(histWidth); // this gets all the widths of all the histograms 
				}
				
		nd.talk("maxWidth = "+maxWidth);		
		// normalize the foci array
		for(i=0; i<npix; i++) {
			histWidth = 255.0*foci[i]/maxWidth;
			outras[i] = (short)(0.49 + histWidth);
			sum += histWidth;
			}
	
		nd.talk("average brightscale = "+sum/npix);	
		return true;	
		}
}