package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.*;  import javax.swing.*;

class GMOColorPicker extends JFrame implements java.awt.event.ActionListener,javax.swing.event.ChangeListener {
	int red=128,green=128,blue=128;
	JButton doneButton;
	JSlider redSlider,greenSlider,blueSlider;
	Color myBlue = new Color(120,120,255);
	JTextField rtf,gtf,btf;
	PickColors parent1=null;
	TwoColorPicker parent2=null; //one of these will be non-null and will have a method
	// called getAllColors() which is called when the "done" button is clicked
	boolean iffirst=false;
	int nomWidth = 225;

	public GMOColorPicker(int xorg, int yorg, String title,PickColors parent,
		int r, int g, int b,JTextField rtf, JTextField gtf, JTextField btf)  { 
        super(title);
        parent1=parent;
        setup( xorg,  yorg, r,  g,  b, rtf,  gtf,  btf);
        }
   
	public GMOColorPicker(int xorg, int yorg, String title,TwoColorPicker parent,
		boolean iffirst,
		int r, int g, int b,JTextField rtf, JTextField gtf, JTextField btf)  { 
        super(title);
        parent2=parent;
        this.iffirst=iffirst;
        setup( xorg,  yorg, r,  g,  b, rtf,  gtf,  btf);
        }
   
   private void setup(int xorg, int yorg,int r, int g, int b,JTextField rtf, 
   					JTextField gtf, JTextField btf)  {
		this.rtf=rtf;
		this.gtf=gtf;
		this.btf=btf;
		red = r; green=g; blue=b;  
        getContentPane().setLayout(new GridLayout(4,1));    
		JPanel reportPanel = new JPanel();
		reportPanel.setLayout(new GridLayout(0,1));
		WidgetMaker wmk = new WidgetMaker(this);
		doneButton=wmk.makeAJButton("click to pick this color");
		repaintDoneButton();
		reportPanel.add(doneButton);
		redSlider = new JSlider(JSlider.HORIZONTAL,0,255,red);
		redSlider.setBackground(Color.red);
		redSlider.addChangeListener(this);
		greenSlider = new JSlider(JSlider.HORIZONTAL,0,255,green);
		greenSlider.setBackground(Color.green);
		greenSlider.addChangeListener(this);
		blueSlider = new JSlider(JSlider.HORIZONTAL,0,255,blue);
		blueSlider.setBackground(myBlue);
		blueSlider.addChangeListener(this);
		getContentPane().add(reportPanel);
		getContentPane().add(redSlider);
		getContentPane().add(greenSlider);
		getContentPane().add(blueSlider);
		setSize(nomWidth,60);
		setLocation(xorg,yorg);
		setVisible(true);		
		}
		
	private void report() {
		rtf.setText(""+red);
		gtf.setText(""+green);
		btf.setText(""+blue);
		repaintDoneButton();
		}
		
	private void repaintDoneButton() {
		Color currentColor = new Color(red,green,blue);
		Color complementColor = new Color(255-red,255-green,255-blue);
		doneButton.setBackground(currentColor);
		doneButton.setForeground(complementColor);
		doneButton.setText("click to pick this color");
		doneButton.updateUI();
		doneButton.repaint();
		}
		
	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
		
	public void actionPerformed(java.awt.event.ActionEvent event) {
		String arg = event.getActionCommand();
		if(arg.equals("click to pick this color")) {
			if(parent1 != null) parent1.getAllColors();
			if(parent2 != null && iffirst) 
				parent2.getFirstColors();
			if(parent2 != null && !iffirst) 
				parent2.getSecondColors();
			doneButton=null;
			redSlider=null;
			greenSlider=null;
			blueSlider=null;
			dispose();
			}
		}
		
	public void stateChanged(javax.swing.event.ChangeEvent e) {
		red = redSlider.getValue();
		green = greenSlider.getValue();
		blue = blueSlider.getValue();
		report();
		} 
}