package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.*;
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import java.io.*;
import java.awt.im.*;

// needs general file size update
class GMORasFuncs extends JFrame{
	RandomAccessFile rasRowFile=null;
	byte [] outrowdata = null;
	static int inputBytesAvailable = -1;
	static String mostRecentInputFileName="";
	static String mostRecentInputPathName="";
	static String mostRecentFileNameShort="";
	     
 	private void encodeHeader(int rows,int cols,int bytesPerPix,byte [] ras) {
		int highByte,lowByte;
		highByte = rows/128; // since I do not need to encode huge ints, use values
							// between 0 ande 127. This skirts Java's hassle of not allowing
							// unsigned byte data types...
		lowByte = rows - 128*highByte;
		ras[0] = (byte)highByte;	ras[1] = (byte)lowByte;
		highByte = cols/128; 
		lowByte = cols - 128*highByte;
		ras[2] = (byte)highByte;	ras[3] = (byte)lowByte;
		ras[4] = (byte)bytesPerPix;
		}

	private int rowsFromHeader(byte [] ras) {
		int highByte = ras[0];
		int lowByte = ras[1];
		return (128*highByte + lowByte);
		}

	private int colsFromHeader(byte [] ras) {
		int highByte = ras[2];
		int lowByte = ras[3];
		return (128*highByte + lowByte);
		}

	private int bytesPerPixFromHeader(byte [] ras) {
		return ((int)ras[4]);
		}
	
    public int [] readGmoRgbRas()  throws IOException {
    	return(readGmoRgbRas(255));
    	}
    	
    public int [] readGmoRgbRas(int alpha) throws IOException {
		RandomAccessFile in;
 		if((in=openReadingFile(this,"GmoRas input file name?")) != null)
		    return readGmoRgbRas(in,alpha);
		else return null;
		}

     public int [] readGmoRgbRas(RandomAccessFile in){
    	return(readGmoRgbRas(in,255));
    	}
    	
    public int [] readGmoRgbRas(RandomAccessFile in, int alpha){
    	/* this returns an image raster with one 4-byte encoding each pixel with this encoding:
    		pix = (alpha<<24) | (red << 16) | (green << 8) | blue;
    		The user supplies the alpha transparency value... 
    		The first int of the array returned is the number of rows in the raster.
    		The second int of the array is the number of columns in the raster.  
    		To "erase" these two pixels, call eraseHeader(int [] ras);   */
    		
		byte [] header = new byte[5];
		int rows=0,cols=0,bytesPerPix=0,kras,kin,red,green,blue,c,r;
		alpha = 255 << 24;
		int picType = 0;
		
		try {
		    if(in.read(header) != 5) {
			talk("failed to read header");
			return null;
			}
		    rows = rowsFromHeader(header);
		    cols = colsFromHeader(header);
		    bytesPerPix = bytesPerPixFromHeader(header);
		    in.seek((long)0);
		    if(inputBytesAvailable != rows*cols*bytesPerPix) {
		    // maybe it's a tiff file. 
			switch (inputBytesAvailable) {
			    case 393892:  
					in.seek((long)8);
					talk("loading 768 by 512 monochrome tiff image file.");
					rows=512;
					cols=768;
					bytesPerPix = 1;
				break;
			   	 	case 393818:  // a monochrome tiff image made by imageConverter... 
					in.seek((long)602);
					talk("loading 768 by 512 monochrome tiff image file.");
					rows=512;
					cols=768;
					bytesPerPix = 1;
					break;
			    case 921780:  
					in.seek((long)8);
					talk("loading 640 by 480 color tiff image file.");
					rows=480;
					cols=640;
					bytesPerPix = 3;
					break;
			    case 1180006: 
					in.seek((long)8);
					talk("reading 768 by 512 color tiff image file.");
					rows=512;
					cols=768;
					bytesPerPix = 3;
					break;
			    case 1181220:  // image converter 768x512 color tiff
					in.seek((long)1572);
					talk("reading 768 by 512 color tiff image file.");
					rows=512;
					cols=768;
					bytesPerPix = 3;
					break;
			    case 1843638: 
					in.seek((long)8);
					talk("reading 1280 by 480 stereo pair color tiff image file.");
					rows=480;
					cols=1280;
					bytesPerPix = 3;
					break;
			    default : 
			    talk("It's not a gmo format file, nor any tiff file");
			    talk("I can deal with. Sorry. It has "+inputBytesAvailable+" bytes");
			    return null;
			    }
			}
			talk("Image has "+rows+" rows and "+cols+" columns with "+bytesPerPix+
					" bytes per pixel");
			byte [] indat = new byte[bytesPerPix*cols];
			int [] ras = new int [rows*cols];
			kras=0;
			for(r=0;r<rows;r++) {
				kin=0;
				if(in.read(indat) != bytesPerPix*cols) {
					talk("failed to read enough data; got only "+r+" rows");
					return null;
					}
				for(c=0;c<cols;c++) {
					if((red = indat[kin++]) < 0 )   red += 256;
					if(bytesPerPix==3) {
						if((green = indat[kin++]) < 0 ) green += 256;
						if((blue = indat[kin++]) < 0 )  blue += 256;
						}
					else {
						green=red;
						blue=red;
						}
					ras[kras++] = alpha | (red << 16) | (green << 8) | blue;
					}
				}
			// put the rows, cols, and bytesPerPix into the first 3 ints of the array we return
			ras[0] = rows;
			ras[1] = cols;
			return ras;
			}
		catch(IOException e) {
	    	talk("Reading GmoRas file failed somehow?? IOException = "+e);
	    	return null;
	    	}
		}

    public String writeJpeg(int rows, int cols, int [] data, int jpgQual) throws IOException {
		String filename = getBaseFileName(this,"name of Jpeg file to write? ");
		if((filename) != null) 
		    writeJpeg(filename, rows, cols, data, jpgQual);
		return filename;
		}

	public void writeJpeg(String filename,int rows,int cols,int [] data,int jpgQual) {
	Jpegger.jpegFromIntRaster(cols,rows, data,  filename);
    }

			
    public void writeJpeg(String filename,int rawrows,int rawcols,int [] data,int jpgQual, 
    			double zoomFac) {
    if(zoomFac>.99 && zoomFac<1.01) { // no need to change image size
    	writeJpeg(filename, rawrows, rawcols, data, jpgQual);
    	return;
    	}
    // else, this outputs a jpeg file that is magnified by a factor = zoomFac
    int rows = (int)(.49 + zoomFac*rawrows);
    int cols = (int)(.49 + zoomFac*rawcols);
    
	int	npix=rows*cols,c, r, rawrow, rawcol, kin,kout,p, pxp, pyp, pxypp,pavg,offset,
		rgbp,rgbpxp,rgbpyp,rgbpxypp,colim = rawcols-1, rowlim=rawrows-1;
	double x,y,xrat,yrat;
	int redmask = (255 << 16);
	int greenmask = (255 << 8);
	int bluemask = 255;
	byte [] outdata = new byte[3*rows*cols]; 

	kout = 0;
	for(r=0;r<rows;r++) 
		for(c = 0; c < cols; c++) {
			x = c/zoomFac;
			y = r/zoomFac;
			rawrow = (int) y;
			rawcol = (int) x;
			offset = rawrow*rawcols + rawcol;
			p = data[offset];			
			pxp = rawcol>=colim? data[offset]: data[offset+1];
			
			if(rawrow < rowlim) offset += rawcols;
			
			pyp = data[offset];			
			pxypp  = rawcol>=colim? data[offset]: data[offset+1];
			
			xrat = (x-rawcol);
			yrat = (y-rawrow);
			
			rgbp = ((p & redmask) >> 16); // blend the red pixel
			rgbpxp = ((pxp & redmask) >> 16);
			rgbpyp = ((pyp & redmask) >> 16);
			rgbpxypp = ((pxypp & redmask) >> 16);
			outdata[kout++] = (byte)(.49+ (rgbp*(1.-xrat)  +  rgbpxp*(xrat) )*(1.-yrat) +
				( rgbpyp*(1.-xrat) + rgbpxypp*(xrat) )*(yrat) );

			rgbp = ((p & greenmask) >> 8); // blend the green pixel
			rgbpxp = ((pxp & greenmask) >> 8);
			rgbpyp = ((pyp & greenmask) >> 8);
			rgbpxypp = ((pxypp & greenmask) >> 8);
			outdata[kout++] = (byte)(.49+ (rgbp*(1.-xrat)  +  rgbpxp*(xrat) )*(1.-yrat) +
				( rgbpyp*(1.-xrat) + rgbpxypp*(xrat) )*(yrat) );
				        
			rgbp = ((p & bluemask)); // blend the blue pixel
			rgbpxp = ((pxp & bluemask));
			rgbpyp = ((pyp & bluemask));
			rgbpxypp = ((pxypp & bluemask));
			outdata[kout++] = (byte)(.49+ (rgbp*(1.-xrat)  +  rgbpxp*(xrat) )*(1.-yrat) +
				( rgbpyp*(1.-xrat) + rgbpxypp*(xrat) )*(yrat) );
			}

	Jpegger.jpegFromRGBByteRaster(cols,rows,outdata,filename) ;
	}

  public void writeGmoRgbRas(int rows, int cols, int [] data) throws IOException {
		RandomAccessFile rasFile;
		if((rasFile=openWritingFile(this,"GmoRas file to write?")) != null) {
		    	writeGmoRgbRas(rasFile, rows, cols, data);
		    	}
		}

   public void writeGmoRgbRas(int rows, int cols, int [] data, String filename) throws IOException {
		RandomAccessFile rasFile;
		if((rasFile=openWritingFile(filename)) != null) {
		    	writeGmoRgbRas(rasFile, rows, cols, data);
		    	}
		}

    public void writeGmoRgbRas(RandomAccessFile out,int rows,int cols,int [] data) {
	int	npix=rows*cols,c, r, kin,kout,p;
	int redmask = (255 << 16);
	int greenmask = (255 << 8);
	int bluemask = 255;
	byte	[] outdata = new byte[3*cols]; // t o conserve RAM, write it one row at a time

	/* write out the .tiff file header */
	    try {
			kin=0;
	    	for(r=0;r<rows;r++) {
				kout=0;
				for(c = 0; c < cols; c++) {
		    		p = data[kin++];
		    		outdata[kout++] = (byte)((p & redmask) >> 16);
		    		outdata[kout++] = (byte)((p & greenmask) >> 8);
		    		outdata[kout++] = (byte)(p & bluemask);
					}
				if(r==0)  // it's the first row. Encode raster size and pixel format in first 5 bytes.
					encodeHeader(rows,cols,3,outdata);
				out.write(outdata);
				}
		outdata=null;
		out.close(); 
		talk("GmoRas file successfully written...");
		}
	catch(IOException e) {
	    talk("GmoRas file write failed somehow");
	    }
    }

   public void writeGmoRgbRasRows(int rows, int cols, int [] onerow, int iwh) {
		int	c, r,kout,p;
		int redmask = (255 << 16);
		int greenmask = (255 << 8);
		int bluemask = 255;

		// iwh == 1 ==> open the file and write the first row, encoding header.
		// iwh ==0 ==> write another line into already opened file
		// iwh ==-1 ==> write the final row and then close the file; 
		if(iwh==1) {
				rasRowFile=openWritingFile(this,"GmoRas file to write?");
				outrowdata = new byte[3*cols]; 
			}
		if(rasRowFile==null) return;
		// write the line
	    try {
			kout=0;
			for(c = 0; c < cols; c++) {
		    	p = onerow[c];
		    	outrowdata[kout++] = (byte)((p & redmask) >> 16);
		    	outrowdata[kout++] = (byte)((p & greenmask) >> 8);
		    	outrowdata[kout++] = (byte)(p & bluemask);
				}
			if(iwh==1)  // it's the first row. Encode raster size and pixel format in first 5 bytes.
					encodeHeader(rows,cols,3,outrowdata);
			rasRowFile.write(outrowdata);
	    	if(iwh== (-1)) {
				rasRowFile.close(); 
				talk("gmoRas file written successwfully");
				}
			}
		catch(IOException e) {
	    	talk("GmoRas file write failed somehow");
	    	}
    }

	static String getBaseFileName(JFrame theJFrame,String prompt) {
		RandomAccessFile outfile;
		FileDialog getFileToWrite = 
		   new FileDialog(theJFrame,prompt, FileDialog.SAVE);
		getFileToWrite.setVisible(true);
		return(getFileToWrite.getDirectory()+"/"+getFileToWrite.getFile());
		}

	static RandomAccessFile openWritingFile(JFrame theJFrame,String prompt) {
		RandomAccessFile outfile;
		FileDialog getFileToWrite = 
		   new FileDialog(theJFrame,prompt, FileDialog.SAVE);
		getFileToWrite.show();
		String filename = getFileToWrite.getDirectory()+"/"+getFileToWrite.getFile();
		if(filename == null) return null;
		return openWritingFile(filename);
		}
		
	static RandomAccessFile openWritingFile(String filename) {
		RandomAccessFile outfile;
		if(filename == null) return null;
		try{
			outfile = new RandomAccessFile(filename,"rw");
			}
		catch (IOException e) {
			talk("openWritingFile("+filename+") failed");
			return null;
			}
		try{outfile.seek((long)0);}
		catch(IOException e) {};
		return outfile;
		}
		
	static RandomAccessFile openReadingFile(JFrame theJFrame,String prompt) {
		RandomAccessFile infile;
		FileInputStream instream; // needed for using the available() method
		FileDialog getFileToRead = 
		   new FileDialog(theJFrame,prompt, FileDialog.LOAD);
		getFileToRead.show();
		mostRecentInputFileName = getFileToRead.getDirectory()+"/"+getFileToRead.getFile();
		mostRecentInputPathName = getFileToRead.getDirectory()+"/";
		mostRecentFileNameShort = getFileToRead.getFile();
		
		inputBytesAvailable = -1;
		if(mostRecentInputFileName == null) return null;
		try{
			instream  = new FileInputStream(mostRecentInputFileName);
			inputBytesAvailable = instream.available();
			talk("file size = "+inputBytesAvailable);
			instream.close();
			infile = new RandomAccessFile(mostRecentInputFileName,"r");
			}
		catch (IOException e) {
			talk("openReadingFile("+prompt+") failed");
			return null;
			}
		try{infile.seek((long)0);}
		catch(IOException e) {};
		return infile;
		}
		
	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
}


