package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.*;  import javax.swing.*;


class GoToMarks extends JFrame implements java.awt.event.ActionListener,java.awt.event.WindowListener {
	ShowPicJPanel picJPanel;
	int nomwidth=160, nomheight=174;
	JButton click2PickMarker,click2MoveMarker,goToMarker,showHideMarks;
	String showMarks="show all marks", hideMarks="hide all marks",
		pickStg="next click selects mark",gotomark="goto mark ",
		moveStg="next click moves ";
	WidgetMaker wmk = new WidgetMaker(this);
	JButton [] gobut = {
		wmk.makeAJButton(gotomark+"00"), 
		wmk.makeAJButton("1",255,255,128),
		wmk.makeAJButton("2",240,255,128),
		wmk.makeAJButton("3",225,255,128),
		wmk.makeAJButton("4",210,255,128),
		wmk.makeAJButton("5",195,255,128),
		wmk.makeAJButton("6",180,255,128),
		wmk.makeAJButton("7",165,255,128),
		wmk.makeAJButton("8",150,255,128),
		wmk.makeAJButton("9",135,255,128),
		wmk.makeAJButton("10",120,255,128)};  
	RGBMarker myMark=null;
	int imyMark=0;
	final static int MOUSE_PICKS_A_MARK = 10;
	final static int MOUSE_MOVES_SELECTED_MARK = 11;
	
    public GoToMarks(ShowPicJPanel picJPanel) {   
    super("goTo");
    this.picJPanel=picJPanel;
    getContentPane().setLayout(new BorderLayout()); 
    setSize(nomwidth, nomheight);
	JPanel goPan,markerJPanel;
	int rr=100,gg=200,bb=255;
	click2PickMarker = wmk.makeAJButton(pickStg);
	click2MoveMarker =  
	    wmk.makeAJButton(moveStg);
	showHideMarks = wmk.makeAJButton(showMarks);

	markerJPanel = new JPanel();
	markerJPanel.setLayout(new GridLayout(0, 1));
	markerJPanel.add(click2PickMarker);
	markerJPanel.add(click2MoveMarker);
	markerJPanel.add(showHideMarks);
	markerJPanel.add(wmk.makeAJButton("tell distance between marks"));
	
	goPan = new JPanel();
	goPan.setLayout(new GridLayout(0, 2));
	for(int i=1;i<11;i++) {
		gobut[i].setLabel(gotomark+i);
		goPan.add(gobut[i]);
		}
	getContentPane().add("North",markerJPanel);
	getContentPane().add("Center",goPan);
	addWindowListener(this);
	wmk=null; // we're done with it
  }

    public void adoptNewMarker(int iwh, RGBMarker myMark) {
	this.myMark=myMark;
	//talk("go panel adopts marker number "+iwh);
	moveStg =("next click moves mark "+iwh); 
	click2MoveMarker.setText(moveStg);
	imyMark=iwh;
	}
	
// catch WindowListener events  
	public void windowActivated(java.awt.event.WindowEvent event) {
		// talk("window activated");
		}
	public void windowDeactivated(java.awt.event.WindowEvent event) {
		//  talk("window Deactivated");
		}
	public void windowOpened(java.awt.event.WindowEvent event) {}
	public void windowIconified(java.awt.event.WindowEvent event) {}
	public void windowDeiconified(java.awt.event.WindowEvent event) {}
	public void windowClosing(java.awt.event.WindowEvent event) {
	    setVisible(false);
		}
	public void windowClosed(java.awt.event.WindowEvent event) {
		}	 

// catch button and menu events...
	public void actionPerformed(java.awt.event.ActionEvent event) {
		String arg = event.getActionCommand();
			if(arg.equals(showMarks)) { // show all marks
				showHideMarks.setText(hideMarks);
				picJPanel.showHideMarks(true);
				}
			if(arg.equals(hideMarks)) { // hide all marks
				showHideMarks.setText(showMarks);
				picJPanel.showHideMarks(false);
				}
			if(arg.equals("tell distance between marks")) { 
				picJPanel.measureMarks(-1);
				picJPanel.parent.show();
				picJPanel.requestFocus();
				}
							    
			if(arg.equals(pickStg)) { // next mouse click picks closest mark
				talk("Next mouse click selects closest mark to mouse location");
				primeMouseForMarkers(MOUSE_PICKS_A_MARK);
				}
							    
			if(arg.equals(moveStg)) {
				talk("Next mouse click moves "+picJPanel.curMarkString+" to mouse location");
				primeMouseForMarkers(MOUSE_MOVES_SELECTED_MARK);
				}
			for(int i=1;i<11;i++) 
				if(arg.equals(gotomark+i)) {
					picJPanel.goToMarker(i);
    				}
    }

   	private void primeMouseForMarkers(int whatTask) {
   		if(picJPanel != null) {
			picJPanel.primeMouseForTask(whatTask);
			picJPanel.parent.show();
			picJPanel.requestFocus();
			}
		}

	public void changeJPanelSize(double ratio) {
		int pwidth = getSize().width;
		int pheight = getSize().height;
		pwidth = (int)(ratio*pwidth +.4999);
		pheight = (int)(ratio*pheight +.4999);
		setSize(pwidth,pheight);
		show();
		}

	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
}

