package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;  
import javax.swing.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;

class GraphicsPs{
	Graphics2D g=null;
	RandomAccessFile psoutFile=null;
	int width=100,height = 100, top=5000,fontscale=120;
	/* want to map (0,0) to (500,7000) and (width,height) to (5600,500)
	x ->  500 + (top*x)/8000
	y -> 7000 - (y*top)/8000 
	where  500 + (top*width)/8000 <= 5600  ==> top <= (5100*8000)/width
	and where 7000 - (height*top)/8000 >= 500 ==> top <= (6500*8000)/height   */
	double [] cosines = null;
	double [] sines = null;
	
	/* When ifWritePostscriptFile==false, this just emulates some of the methods 
	of the Graphics() object by relaying method calls to g. When ifWritePostscriptFile==true, 
	then, in addition to relaying calls to the Graphics object g, this opens and writes
	a postscript file defining the picture (after asking you to supply the name of the file).
	
	As of November 2002, I use Graphics2D instead of Graphics objects. */
	public GraphicsPs(int wd, int ht,Graphics2D g, boolean ifWritePostscriptFile) {
		this.g=g;	
		if(ifWritePostscriptFile) {
			psoutFile = openWritingFile("Pick file to write postscript pic to");
			if(psoutFile == null) {
				talk("write failed???");
				return;
				}
			}
		makePSheader(wd, ht);
		}
		
	private void makePSheader(int wd, int ht) {
		width=wd;
		height=ht;
		if((top=(5100*8000)/width) > (6500*8000)/height) 
			top=(6500*8000)/height;
		if(psoutFile!=null) {
			fontscale = 90; // just a guess			
			writeAString(" .09 .09 scale\nnewpath\n/Helvetica findfont "+fontscale+" scalefont setfont\n");
			writeAString(" 6 setlinewidth\n");			
			}
		}
		
	public void finish() {
		if(psoutFile!=null) {
			writeAString(" showpage\n");
			closeOutputFile(psoutFile);
			}
		}
		
	private double relinten(int eightbit) {
		double z = (double)(eightbit)/2.550;
		int i = (int)(z);
		return .01*(double)(i);
		}
		
	public void newColor(String colorname) {
		if(colorname.equals("red")) { newColor(255,0,0); return;}
		if(colorname.equals("green")) { newColor(0,255,0); return;}
		if(colorname.equals("blue")) { newColor(0,0,255); return;}
		if(colorname.equals("cyan")) { newColor(0,255,255); return;}
		if(colorname.equals("yellow")) { newColor(255,255,0); return;}
		if(colorname.equals("magenta")) { newColor(255,0,255); return;}
		if(colorname.equals("gray")) { newColor(80,80,80); return;}
		if(colorname.equals("darkGray")) { newColor(64,64,64); return;}
		if(colorname.equals("lightGray")) { newColor(128,128,128); return;}
		if(colorname.equals("black")) { newColor(0,0,0); return;}
		}
		
	public void drawImage(Image rawImage, int xorig, int yorig, int wd,int ht, ShowPicJPanel parent)	{
		g.drawImage(rawImage, xorig, yorig, wd, ht, parent);
		// just echo this until I figure out how to draw pictures in postscript.
		}
		
	public void drawImage(Image rawImage, int xorig, int yorig, int wd,int ht, SegTowerCanvas parent)	{
		g.drawImage(rawImage, xorig, yorig, wd, ht, parent);
		// just echo this until I figure out how to draw pictures in postscript.
		}
		
	public void fillBackgroundRectangle(int rd, int gr, int bl) {		
		g.setColor(new Color(rd,gr,bl));
		int y = 0;
		int x = 0;
		g.fillRect(x,y,width,height); 
		talk("erased background on cam diagram with width="+width+" and height ="+height);
		if(psoutFile!=null) {
				if(rd==0 && gr==0 && bl==0) // if it's black, make it white
					writeAString(" newpath\n 1.0 1.0 1.0  setrgbcolor  %turned black background to white\n");
				else 
					writeAString(" newpath\n"+relinten(rd)+" "+relinten(gr)+" "+relinten(bl)+" setrgbcolor\n");
				y += height;
				x = 500 + (top*x)/8000;
				y = 7000 - (y*top)/8000;
				writeAString(x+" "+y+" "+((top*width)/8000)+" "+((top*height)/8000)+" rectfill\n");
				}
		}
		
			
	public void newColor(int rd, int gr, int bl) {
		g.setColor(new Color(rd,gr,bl));
		if(psoutFile!=null) {
			if(rd==gr && gr == bl) { // it's a shade of 
				writeAString(" newpath\n"+relinten(rd)+" setgray\n");
				}
			else
				writeAString(" newpath\n"+relinten(rd)+" "+relinten(gr)+" "+relinten(bl)+" setrgbcolor\n");
			}
		}
	
	public void fillCenteredRect(int cenx,int ceny,int wd,int ht) {
		fillRect(cenx-wd/2,ceny-ht/2,wd,ht);
		}
	
	public void drawCenteredRect(int cenx,int ceny,int wd,int ht) {
		drawRect(cenx-wd/2,ceny-ht/2,wd,ht);
		}
	
	public void fillRect(int x,int y,int wd,int ht) {
		g.fillRect(x,y,wd,ht);
		if(psoutFile!=null) {
			makeRect(x,y,wd,ht);
			writeAString(" rectfill");
			}
		}
				
	public void drawRect(int x,int y,int wd,int ht) {
		g.drawRect(x,y,wd,ht);
		if(psoutFile!=null) {
			makeRect(x,y,wd,ht);
			writeAString(" rectstroke");
			}
		}		
		
	private void makeRect(int x,int y,int wd,int ht) {			
				y += ht;
	// x ->  500 + (top*x)/8000   and   y -> 7000 - (y*top)/8000 
				x = 500 + (top*x)/8000;
				y = 7000 - (y*top)/8000;
				writeAString("newpath "+x+" "+y+" "+((top*wd)/8000)+" "+((top*ht)/8000));
			}

	public void drawPolygon(int [] xv, int [] yv, int nverts) {
		g.drawPolygon(xv,yv,nverts);		
		if(psoutFile!=null) {
			makePolygon(xv,yv,nverts);
			writeAString(" stroke");
			}
		}
			
	public void fillPolygon(int [] xv, int [] yv, int nverts) {
		g.fillPolygon(xv,yv,nverts);		
		if(psoutFile!=null) {
			makePolygon(xv,yv,nverts);
			writeAString(" fill");
			}
		}
			
	private void makePolygon(int [] xv, int [] yv, int nverts) {
		int i,x,y;
		x = 500 + (top*xv[0])/8000;
		y = 7000 - (yv[0]*top)/8000;
		writeAString(" newpath "+x+" "+y+" moveto");
		for(i=1;i<nverts;i++) {
			x = 500 + (top*xv[i])/8000;
			y = 7000 - (yv[i]*top)/8000;
			writeAString(x+" "+y+" lineto");
			}
		writeAString(" closepath \n");
		}
		
	public void drawCenteredOval(int xcen, int ycen, int wd, int ht) {
		drawOval(xcen-wd/2,ycen-ht/2,wd,ht);
		}
	
	public void drawOval(int left, int tip, int wd, int ht) {
		g.drawOval(left,tip,wd,ht);
		if(psoutFile!=null) {
			makeOval(left,tip,wd,ht);
			writeAString(" stroke \n");
			}
		}
	
	public void fillCenteredOval(int xcen, int ycen, int wd, int ht) {
		fillOval(xcen-wd/2,ycen-ht/2,wd,ht);
		}
	
	public void fillOval(int left, int tip, int wd, int ht) {
		g.fillOval(left,tip,wd,ht);
		if(psoutFile!=null) {
			makeOval(left,tip,wd,ht);
			writeAString(" fill \n");
			}
		}
	
	private void makeOval(int left, int tip, int wd, int ht) {
	/* the points on the ellipse lie on the locus
	x(t) = xctr + xrad*cos(t)
	y(t) = yctr + yrad*sin(t), for t between 0 and 2*3.14159 */
		int i,x,y,npts = 200;
		double xfl,yfl,t=0,tinc;
		double radx = .5*(double)wd;
		double rady = .5*(double)ht;
		double xctr = (double)left + radx;
		double yctr = (double)tip + rady;
		tinc = (2.0*3.14159)/(double)(npts);
		if(cosines==null) { // generate the sines and cosines arrays
			cosines = new double[npts+1];
			sines = new double[npts+1];
			for(i=0;i<npts;i++) {
				t = i*tinc;
				cosines[i]=Math.cos(t);
				sines[i]=Math.sin(t);
				}
			}
		xfl = xctr + radx;
		yfl = yctr;
		writeAString("% this is an oval\n newpath "+(int)(500. + (top*xfl)/8000.)+" "+
			(int)(7000. - (yfl*top)/8000.)+" moveto");
		for(i=1;i<npts;i++) {
			xfl = xctr + radx*cosines[i];
			yfl = yctr + rady*sines[i];
			x = (int)(.499 + top*xfl);
			x = 500 + x/8000;
			y = (int)(.499 + top*yfl);
			y = 7000 - y/8000;
			writeAString(x+" "+y+" lineto ");
			}
		writeAString(" closepath \n");
		}
				
	public void drawLine(int xfrom,int yfrom,int xto,int yto) {
		g.drawLine(xfrom,yfrom,xto,yto);
		if(psoutFile!=null) {
			xfrom = 500 + (top*xfrom)/8000;
			yfrom = 7000 - (yfrom*top)/8000;
			xto = 500 + (top*xto)/8000;
			yto = 7000 - (yto*top)/8000;
			writeAString(" newpath "+xfrom+" "+yfrom+" moveto "+xto+" "+yto+" lineto stroke");
			}
		}
	
	public void drawString(String theStg, int x, int y) {
		g.drawString(theStg, x, y);
		if(psoutFile!=null) {
			x = 500 + (top*x)/8000;
			y = 7000 - (y*top)/8000;
			writeAString("newpath\n"+x+" "+y+" moveto\n");
			writeAString("("+theStg+") show\n");
			}; 
		}
							
	private void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
	
	public void writeAString(String stg) {
		byte [] buf = new byte[stg.length()+1];
		stg.getBytes(0,stg.length(),buf,0);
		buf[stg.length()] = 10;
			try {
			psoutFile.write(buf);
			}
		catch (IOException ioex) {
			talk("write to file failed: "+ioex);
			}
		}
		
	private RandomAccessFile openWritingFile(String prompt) {
		RandomAccessFile outfile;
		FileDialog getFileToWrite = 
		   new FileDialog(new JFrame(), prompt, FileDialog.SAVE);
		getFileToWrite.show();
		String filename = getFileToWrite.getDirectory()+"/"+getFileToWrite.getFile();
		if(filename == null) return null;
		try{
			outfile = new RandomAccessFile(filename,"rw");
			}
		catch (IOException e) {
			talk("openWritingFile("+prompt+") failed");
			return null;
			}
		try{outfile.seek((long)0);}
		catch(IOException e) {};
		return outfile;
		}

	private  void closeOutputFile(RandomAccessFile inoutf) {
		try{inoutf.close();} catch(IOException e) {
			talk("closeOutputFile("+inoutf+") failed ???");};
		}		

}