package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;  import javax.swing.*;
import java.awt.image.*;
import java.util.*;
import java.lang.*;
import java.io.File;

/* the center of this leaf is at In absolute units....
    x coord = parent.cg[0] - curStack.Xshifts[chan] + myOffset[0] 
    y coord = parent.cg[1] - curStack.Yshifts[chan] + myOffset[1]
    z coord = parent.cg[2] - curStack.Zshifts[chan] + myOffset[2]
		    */
public class MarkerLeaf {
BioradCanvas curStack;
RGBMarker parent;
Color glow;
double [] myOffset = {0., 0., 0.};
int chan = 0;
double wide = 10.;
double high = 10.;
int myShape=0; // 0->rectangle, 1->diamond, 2->cross, 3->X, 4->oval, 5-->void
boolean ifVisible=false;
boolean ifSelected=false;

    public MarkerLeaf(RGBMarker parent) {
	this.parent=parent;
	}

    public boolean isVoid() {
	return myShape==5;
	}

     public boolean isActive() {
	return (parent.isActive() && !isVoid());
	}

    public void setProps(Color glow, int chan) {
	this.glow =glow;
	this.chan = chan;
	}

   public void select(boolean ifsel) {
	ifSelected=ifsel;
	}

    public void moveRel(double dx,  double dy,  double dz) {
	myOffset[0] += dx;
	myOffset[1] += dy;
	myOffset[2] += dz;
	stayInBox();
	}

    public void moveOffsetsToHitMouseLoc(int mx,  int my) {
	double [] mouseat = {(double)mx, (double)my, (double)(curStack.curJFrame)};
	curStack.unScaleCoords(mouseat);
	double [] myloc = tellAbsLoc();
	if(myloc==null) return;
	myOffset[0] += (mouseat[0]-myloc[0]);
	myOffset[1] += (mouseat[1]-myloc[1]);
	myOffset[2] += (mouseat[2]-myloc[2]);
	stayInBox();
	}

    public double [] tellAbsLoc() {
    if(parent==null) {
    		talk("MarkerLeaf finds parent==null");
    		return null;
    		}
    		
    if(parent.cg==null) {
    		talk("MarkerLeaf finds parent.cg==null");
    		return null;
    		}
        
     if(curStack==null) {
    		talk("MarkerLeaf finds curStack==null");
    		return null;
    		}     
     
     if(curStack.Xshifts==null) {
    		talk("MarkerLeaf finds curStack.Xshifts==null");
    		return null;
    		}     
     
     if(curStack.Yshifts==null) {
    		talk("MarkerLeaf finds curStack.Yshifts==null");
    		return null;
    		}     
     
     if(curStack.Zshifts==null) {
    		talk("MarkerLeaf finds curStack.Zshifts==null");
    		return null;
    		}     
     
      if(myOffset == null) {
     		talk("MarkerLeaf finds myOffset==null");
    		return null;
    		}     
    		
	double [] xyz = {parent.cg[0] - curStack.Xshifts[chan] + myOffset[0],  
	    parent.cg[1] - curStack.Yshifts[chan] + myOffset[1], 
		parent.cg[2] - curStack.Zshifts[chan] + myOffset[2]};
	return xyz ;	
	}

    public double [] tellOffsets() {
	return myOffset ;	
	}

    public void stayInBox() {
	double [] xyz = tellAbsLoc();
	if(xyz==null) return;
	for(int i=0;i<3;i++)
	    if(xyz[i]<0.) myOffset[i] -= xyz[i];
	if(xyz[0]>curStack.parent.fullWidth-1) 
		myOffset[0] += (curStack.parent.fullWidth-1-xyz[0]);
	if(xyz[1]>curStack.parent.fullHeight-1) 
		myOffset[0] += (curStack.parent.fullHeight-1-xyz[0]);
	if(xyz[2] >= (curStack.NJFrames-1))
	     myOffset[2] += (curStack.NJFrames-1-xyz[2]);
	}

    public void nullOffsets() {
	for(int i=0; i<3;i++) myOffset[i]=0.;
	}

    static void talk(String stg) { // write a line on stdout
	System.out.println(stg);
	}

   public void draw(Graphics g, int iwh,  int zsect, int xorg, int yorg) {
   // iwh==0 means draw if you are visible and your z is close enough to zsect.
   // iwh==1 means draw (visible of not) if your z is close enough to zsect.
   // iwh==2 means unconditional draw.
	if(iwh==0 && !ifVisible) return;
	if(curStack.singleChan>=0 && curStack.singleChan != chan)
		return;  // when drawing a single color, draw only leafs in that channel
	double [] xyz = tellAbsLoc();
	if(xyz==null) return;
	int x = (int)(xyz[0]+.4999);
	int y = (int)(xyz[1]+.4999);
	int z = (int)(xyz[2]+.499);
	int width, height;
	switch (iwh) {  // when arrive here visability no longer matters.
	    case 0: 
	    case 1: 
	    	int delz = (int)xyz[2]-zsect;
	    	if(delz<0) delz=-delz;
			if(delz > curStack.tooZfar2Draw) break;
	    case 2: default:
			width = (int)(wide+.4999);
			height = (int)(high+.4999);
			g.setColor(glow);
			drawMarker(g, x+xorg, y+yorg, width, height, false);
			if(ifSelected)  // draw so as to highlight
			    drawMarker(g, x+xorg, y+yorg, width, height, true);
				
			if(parent.parent.ifDrawZNumbers) {
				width += 10;  height += 10;
			    switch(chan) {
				case 0 : //I'm the red channel
				    y -= height;
				    break;
				case 1 : //I'm the green channel
				    x += (4+width);
				    break;
				case 2 : //I'm the blue channel
				    y += height;
				    break;
				}
			    width = (int)( xyz[2] - parent.cg[2]);
			    g.drawString(""+width, x-4+xorg, y+4+yorg);
			    }
			break;
	    }
	}

    private void centralDot(Graphics g, int x,  int y) {
	g.drawLine(x-1, y, x+1, y);
 	g.drawLine(x, y-1, x, y+1);
    }
	
    private void drawMarker(Graphics g, int x,  int y,  int width,  int height,
	    boolean ifHighLight) {
	if(ifHighLight) {
	    switch(myShape) {
		case 0: // highlight marker with a bigger rectangle
		    width += 2;	    height +=2;
		    g.drawLine(x-width, y-height, x+width, y-height);
		    g.drawLine(x+width, y-height, x+width, y+height);
		    g.drawLine(x+width, y+height, x-width, y+height);
		    g.drawLine(x-width, y+height, x-width, y-height);
		    break;
		case 1: // highlight marker with a bigger diamond
		    width += 2;	    height +=2;
		    g.drawLine(x-width, y, x, y-height);
		    g.drawLine(x, y-height, x+width, y);
		    g.drawLine(x+width, y, x, y+height);
		    g.drawLine(x, y+height, x-width, y);
		    break;
		case 2: // highlight the marker with a diamond
		    g.drawLine(x-width, y, x, y-height);
		    g.drawLine(x, y-height, x+width, y);
		    g.drawLine(x+width, y, x, y+height);
		    g.drawLine(x, y+height, x-width, y);
		    break;
		    
		case 3: // highlight the marker with a rectangle
		    g.drawLine(x-width, y-height, x+width, y-height);
		    g.drawLine(x+width, y-height, x+width, y+height);
		    g.drawLine(x+width, y+height, x-width, y+height);
		    g.drawLine(x-width, y+height, x-width, y-height);
		    break;
		case 4: // highlight marker with a bigger oval
		    width +=2;	    height+=2;
		    g.drawOval(x-width, y-height, 2*width, 2*height);
		    break;
		case 5:
		default: // void marker...
		    break;
		}
	    }
	else {
	    switch(myShape) {
		case 0: // draw marker as a rectangle
		    g.drawLine(x-width, y-height, x+width, y-height);
		    g.drawLine(x+width, y-height, x+width, y+height);
		    g.drawLine(x+width, y+height, x-width, y+height);
		    g.drawLine(x-width, y+height, x-width, y-height);
		    centralDot(g, x, y);
		    break;
		case 1: // draw marker as a diamond
		    g.drawLine(x-width, y, x, y-height);
		    g.drawLine(x, y-height, x+width, y);
		    g.drawLine(x+width, y, x, y+height);
		    g.drawLine(x, y+height, x-width, y);
		    centralDot(g, x, y);
		    break;
		case 2: // draw the marker as a +
		    g.drawLine(x, y-height, x, y+height);
		    g.drawLine(x-width, y, x+width, y);
		    break;
		    
		case 3: // draw the marker as an X
		    g.drawLine(x-width, y-height, x+width, y+height);
		    g.drawLine(x-width, y+height, x+width, y-height);
		    break;
		case 4: // draw marker as an oval
		    g.drawOval(x-width, y-height, 2*width, 2*height);
		    centralDot(g, x, y);
		    break;
		case 5:
		default: // void marker...
		    break;
		}
	  }
	}
}
