package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.*;

/**
 * NiceBalls.java
 *
 * @author Created by G Odell june 2006
 */

import java.util.*;
import java.awt.*;  import javax.swing.*;
/* a segmentation filter that tries to replace elliptical patches of pixels with ellipsoids with best major
and minor axes, and best rotation angle. 
*/
class NiceBalls { 
	SegmenterNode nd=null;
	
	
	public NiceBalls(SegmenterNode nd) {
		this.nd=nd;
		} 

	public void tellWhatYouDo() {
		nd.talk("The NiceBalls filter filter has one black/white input raster which it first partitions into disjoint patches of pixels that are on contiguously. It has two parameters, maxrad & passingScore. Every patch of contiguously on pixels that lie within a circle of radius <= maxrad is replaced by a circular ball of on pixels whose rim is set to black. The filter works its way up from small trial balls to balls of radius maxrad, using the smallest possible ball each contiguous patch of on pixels for which the per-pixel score exceeds passingScore. The pixels that cannot be replaced by balls are left alone.");
		}

	
static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
public boolean doit(short [] red,int NCOLS,int NROWS,int maxrad, double passingScore, short [] grayscale, int grayrows, int graycols, short [] outras) {
int row, col, i, npix, pararclen,
    nswarms, nspots,  spotpoints, swarmpoints; 

talk("NiceBalls.doit(maxrad="+maxrad+",  passingScore="+passingScore);
double radius;
long nvoxon, nvoxoffed;
SwarmOfPoints firstSwarm=null,lastSwarm=null,aswarm,bswarm;
boolean ifinit=false;
Ball3D ball=null;	

	nswarms=0;
	nspots=0;
	nvoxon=0;
	nvoxoffed=0;
	npix=NCOLS*NROWS;  
	short [] ras = new short[npix];
	for(i=0;i<npix;i++) {
		ras[i]=0;
		outras[i]=0;
		}

    row=0;
    col=0; 
	while(row < NROWS) {
		while(col <NCOLS) {
	    if(israspixelon(row, col, red, NCOLS, NROWS) ) { 
			    /* find a voxel that is on */ 
			aswarm = new SwarmOfPoints(red,NCOLS,NROWS,col,row,ras);
			aswarm.collectConnectedSwarm(); // transfers points in aswarm from red[] to ras[]
			if(firstSwarm==null) { // this is the first swarm found
				firstSwarm=aswarm;
				lastSwarm=aswarm;
				}
			else {
				lastSwarm.nextSwarm=aswarm;
				lastSwarm=aswarm;
				}
			}
			++col;
			}
		col=0;
		++row;
		} // when fall through, we have found all connected pieces.
		// ras[] has all the points in it, red[] has none. Restore red[] 
	for(i=0;i<npix;i++)
		red[i]=ras[i];
    ball = new Ball3D(); 
          	
	aswarm = firstSwarm; // now process the connected swarms.
	// if a swsarm can be replaced by a mathematical ball, do so
	// else just transfer it to the outras array.
	double [] ballscore = {0,100,100,0,3,4,1,2} ;// on return = {score,xcen,ycen,angle,kpars[0], kpars[1], kpars[2],...}
	KernelConvolver.setShortRaster(graycols, grayrows, grayscale);
/*	for(int nnn=0;nnn<200;nnn++) { // paint many randomly sized and oriented ellipses just to see that they are what I expect
		 ballscore[0]=.44;
		 ballscore[1]=(int)(NCOLS*Math.random());
		 ballscore[2]=(int)(NROWS*Math.random());
		 ballscore[3]=360*Math.random();
		 ballscore[4]=4+15*Math.random();
		 ballscore[5]=3+16*Math.random();		 
		 ball.paintOnOutRas(outras, NCOLS, NROWS, ballscore);
		 } */
	while(aswarm != null) {
		if(aswarm.getLocations() <= 2*maxrad && (ballscore = ball.ballQ(aswarm, maxrad, passingScore,  false)) != null) {
          				// convert this swarm to a round dot 
			    spotpoints = ball.paintOnOutRas(outras, NCOLS, NROWS, ballscore);
				++nspots;
		    	nvoxon += spotpoints;
		    	nvoxoffed += aswarm.numPoints-spotpoints;
				aswarm = aswarm.dissolve();
				}
			else { // just copy the swarm to outras
		    	nvoxon += aswarm.numPoints;
		    	++nswarms;
				aswarm = aswarm.migrateAndDissolve(ras, outras);
				} 
		}		
		
	tellResults(nswarms, nspots, nvoxon, nvoxoffed);
	return true;
	}

	
	private boolean israspixelon(int y, int x, short [] ras, int NCOLS, int NROWS) {
    	if(x<0 || x>=NCOLS) return false;
    	if(y<0 || y>=NROWS) return false;
    	if(ras[y*NCOLS+x]>127) return true;
    	else return false;    
		}
	

	private void tellResults(int nswarms, int nspots, long nvoxon, long nvoxoffed) {
		nd.talk("NiceBalls(): "+nswarms+" large swarms found that could not be converted....");	
		nd.talk("NiceBalls(): "+nspots+" swarms turned to round spots.");
		nd.talk("NiceBalls(): "+nvoxon+" voxels on in output mask");
		if(nvoxoffed > 0)
			nd.talk("NiceBalls(): "+nvoxoffed+" voxels in input mask were eliminated");
		else 
			nd.talk("NiceBalls(): "+(-nvoxoffed)+" voxels added to those on in input mask");
		}
		
}

