package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.*;
import java.util.*;
import java.awt.image.*;
import java.awt.*;  import javax.swing.*;

class PainterColorPickCanvas extends Canvas implements  java.awt.event.MouseListener {
	PainterColorPicker parent=null;
	int lastX=0;
	int lastY=0;
	int mouseStartX = 0, xrel=0, yrel=0, mouseStartY = 0;
	int edgeLen=181;
	int [] picras = null;
	int alpha = 255<<24;
	double hueAngle=0;
	
	double HUE = .5;
	double CurrentDrawnHue=-1;
	double CHROMA=1.;
	double WHITENESS=1.;
	
	double pi = 3.14159;
	int xcen = edgeLen/2, ycen = edgeLen/2;
	int ringOuterRadius = edgeLen/2 - 1;
	int ringInnerRadius = ringOuterRadius-15;
	int ringRadius = (ringOuterRadius + ringInnerRadius)/2;
	int ringOuterRadiusSq = ringOuterRadius*ringOuterRadius, 
		ringInnerRadiusSq = ringInnerRadius*ringInnerRadius;
	int hotTipX = ringInnerRadius-2;
	// the right-facing saturated color tip of the triangle is at (hotTipX,0)
	int pureGrayX = -(int)(.7071*ringInnerRadius);
	int pureGrayY = (int)(.7071*ringInnerRadius);
	// the pure gray left border of the triangle runs from (pureGrayX,pureGrayY) to (pureGrayX,-pureGrayY)

	public PainterColorPickCanvas(PainterColorPicker parent, int r, int g, int b){
		int i;
		this.parent=parent;
		edgeLen = parent.nomWidth;
		xcen = edgeLen/2;
		ycen = edgeLen/2;
		ringOuterRadius = edgeLen/2 - 1;
		ringInnerRadius = ringOuterRadius-15;
		ringRadius = (ringOuterRadius + ringInnerRadius)/2;
		ringOuterRadiusSq = ringOuterRadius*ringOuterRadius;
		ringInnerRadiusSq = ringInnerRadius*ringInnerRadius;
		hotTipX = ringInnerRadius-2;
		pureGrayX = -(int)(.7071*ringInnerRadius);
		pureGrayY = (int)(.7071*ringInnerRadius);
	// the pure gray left border of the triangle runs from (pureGrayX,pureGrayY) to (pureGrayX,-pureGrayY)
		lastX=0; lastY=0;
		double [] hcw = parent.hcwFromRGBvals(r, g, b);
		setSize(edgeLen,edgeLen);
		addMouseListener(this);
		setNewHCW(hcw[0],hcw[1],hcw[2]);
   		}
   	
	private void updatePicras() {
		double saveHue = HUE;
		double saveChroma=CHROMA;
		double saveWhiteness = WHITENESS;
		double wh,ch;
		int ybot,ytop,x,y;
		int [] darkgray = {40,40,40};
		int [] lightgray = {100,100,100};
		int grayback=10;
		
		if(picras==null) { // first call. Draw background and hue rim
			picras = new int[(edgeLen)*(edgeLen)];
			for(x=0; x<edgeLen; x++)
				for(y=0; y<edgeLen; y++) 
					if(mouseInRingQ(x-edgeLen/2, edgeLen/2 - y)) {
						putpix(x, y, parent.RGBfromHCW(HUE, 1.0, 1.0));
						}
					else 
						putpix(x, y, 15 + x/3);
					}
			
		// update the triangle. The interior of the hue rim never changes.
		HUE = saveHue;
		if(HUE != CurrentDrawnHue) {
		  	CurrentDrawnHue = HUE;
			for(x=0; x<edgeLen; x++)
				for(y=0; y<edgeLen; y++) 
					if(mouseInTriangleQ(x-edgeLen/2, edgeLen/2 - y) )
						putpix(x, y, parent.RGBfromHCW(HUE, CHROMA, WHITENESS));
			}
		// restore original hcw
		CHROMA = saveChroma;
		WHITENESS = saveWhiteness;

		// paint the currently selected color into the outside of the hue rim.
		int [] rgb = parent.RGBfromHCW(HUE, CHROMA, WHITENESS);
		int pack = alpha | rgb[0]<<16 | rgb[1]<<8 | rgb[2];
		int halfLen = edgeLen/2;
		for(x=0; x<edgeLen/2; x++)
			for(y=0; y<edgeLen/2; y++) // it's symmetric only need to do one quarter
				if(outsideRing(x, y)) {// it's symmetric only need to do one quarter
					picras[(y+halfLen)*edgeLen + halfLen + x] = pack;
					picras[(halfLen-y)*edgeLen + halfLen + x] = pack;
					
					picras[(y+halfLen)*edgeLen + halfLen - x] = pack;
					picras[(halfLen-y)*edgeLen + halfLen - x] = pack;
					}
		}
		
	private void putpix(int x, int y, int [] rgb) {
		x = y*edgeLen+x; // the offset into the linear array representing a square grid
		picras[x] = alpha | rgb[0]<<16 | rgb[1]<<8 | rgb[2];
		}
			
	private void putpix(int x, int y, int gray) {
		x = y*edgeLen+x; // the offset into the linear array representing a square grid
		picras[x] = alpha | gray<<16 | gray<<8 | gray;
		}
			
	public void update(Graphics2D g) {
		paint(g);
		}
		
	private void setNewHCW(double h,double c,double w) {
		HUE=h;
		CHROMA=c;
		WHITENESS=w;
		parent.setNewColorFromHCW(HUE,CHROMA,WHITENESS);
		repaint();
		}
			
	public void paint(Graphics g) {
		//talk("This should never happen. PainterColorPickCanvas.paint() called");
		paint2D((Graphics2D)g);
		}
		
	public void update(Graphics g) {
		//talk("PainterColorPickCanvas.update() called. Should never happen");
		paint2D((Graphics2D)g);
		}

	public void update2D(Graphics2D g) {
		paint2D(g);
		}

	public void paint2D(Graphics2D g,  boolean ifVirgin) {
	    paint2D(g);
	    }
	    	
	public void mouseClicked(java.awt.event.MouseEvent event) {
		// talk("Mouse Clicked at ("+event.getX()+", "+event.getY()+")");
		}
		
	public void mousePressed(java.awt.event.MouseEvent event) {
		    mouseStartX = event.getX();
		    mouseStartY = event.getY();
		    if(mouseStartX>=edgeLen) mouseStartX=edgeLen-1;
		    if (mouseStartX<0) mouseStartX=0;
		    if(mouseStartY>=edgeLen) mouseStartY=edgeLen-1;
		    if (mouseStartY<0) mouseStartY=0;
		}		
		
	public void mouseEntered(java.awt.event.MouseEvent event) {
		// talk("Mouse entered at ("+event.getX()+", "+event.getY()+")");
		}
		
	public void mouseExited(java.awt.event.MouseEvent event) {
		// talk("Mouse exited at ("+event.getX()+", "+event.getY()+")");
		}
		
	public void mouseReleased(java.awt.event.MouseEvent event) {	
		xrel = mouseStartX-edgeLen/2;
		yrel = edgeLen/2-mouseStartY;
		if(outsideRing(xrel, yrel)) {
			parent.pickCurrentColor();
			return;
			}
		// talk("mouse up at ("+xrel+","+yrel+") rel coords, with chroma="+CHROMA+" and HUE="+WHITENESS);
		if(mouseInRingQ(xrel,yrel)) {
			// talk("Mouse click occurred in hue-selection ring at angle "+hueAngle+" ==> hue="+HUE);
			// this gets us a new hue
			setNewHCW(HUE,CHROMA,WHITENESS);
			repaint();
			return;
			}
			
		if(mouseInTriangleQ(xrel,yrel)) {
			/* talk("Mouse click occurred in triangle giving chroma= "+CHROMA+" and whiteness = "+WHITENESS+
				" while HUE=="+HUE); */
			// this gets new chroma and whiteness
			setNewHCW(HUE,CHROMA,WHITENESS);
			repaint();
			return;
			}
		}
	
	private boolean outsideRing(int xr, int yr)	 {
		if(xr*xr + yr*yr > ringOuterRadiusSq) 
			return true;
		else
			return false;
		}
		
	private boolean mouseInRingQ(int xr, int yr) {
		int radsq = xr*xr + yr*yr;
		if(radsq > ringOuterRadiusSq || radsq < ringInnerRadiusSq) 
			return false;			
		double rad = Math.sqrt((double)radsq);
		/* have xr = rad*cos(theta) and yr = rad*sin(theta) */
		if(Math.abs(xr)>.0001) {
			hueAngle = Math.atan((double)yr/(double)xr);
			// atan() returns angles between -pi/2 and pi/2
			if(xr>0) {
				if(yr<0) hueAngle += (2*pi);
				}
			else {
				if(yr<0)
					hueAngle += pi;
				else
					hueAngle = hueAngle + pi;
					}
			}
		else {
			if(yr>0) 
				hueAngle = pi/2;
			else
				hueAngle = -pi/2;
			}
		hueAngle = 180.*hueAngle/pi;
		HUE = (360. - hueAngle)/360 - 0.25;
		if(HUE < 0)
			HUE += 1.0;
		if(HUE > 1.)
			HUE = 1.0;
		return true;
		}
		
	private boolean mouseInTriangleQ(int xr, int yr) {
		if(xr < pureGrayX || xr > hotTipX)
			return false;		
		double ch = (double)(xr-pureGrayX)/(double)(hotTipX-pureGrayX);
		/* the equation of the triangle's top border is 
		   yrtop = pureGrayY*(1. - (xr-pureGrayX)/(hotTipX-pureGrayX) ) = pureGrayY*(1. - ch)
		   
		   and the equation of the triangle's bottom border is 	
		   yrtop = pureGrayY*(-1. + (xr-pureGrayX)/(hotTipX-pureGrayX) )  = pureGrayY*(-1. + ch)
		   			*/
		double ybot = (double)pureGrayY*(-1. + ch);
		double ytop = (double)pureGrayY*(1. - ch);
		if(yr > ytop || yr < ybot)
			return false;
		WHITENESS = ((double)yr - ybot)/(ytop-ybot);
		CHROMA = ch;
		return true;
		}
				
	public void paint2D(Graphics2D g) {
		updatePicras(); // this fills in the background, the hue circle and the triangle in the picras
			// array which we copy into the canvas, then overlay the markers of what the hue is and
			// where we are in the whiteness triangle.
		Image rawImage = createImage(new MemoryImageSource(edgeLen, edgeLen,
			 picras, 0, edgeLen));
		g.drawImage(rawImage,0,0,edgeLen,edgeLen,parent);
		// draw marker locating hue
		double huan = 2.0*pi*(1.0-HUE);
		// reversed the sense
		// 0 occurs at six o'clock, so rotate it by pi/2
		huan -= pi/2;
		int xr = (int)(.499+ Math.cos(huan)*ringRadius);		
		int yr = (int)(.499+ Math.sin(huan)*ringRadius);	
		g.setColor(Color.white);
		drawRelCircle(g, xr, yr, 4);	
		g.setColor(Color.black);
		drawRelCircle(g, xr, yr, 6);
		
		// draw marker locating whitness...pureGrayX hotTipX
		xr = (int)(pureGrayX + CHROMA*(hotTipX - pureGrayX));
		double ybot = (double)pureGrayY*(-1. + CHROMA);
		double ytop = (double)pureGrayY*(1. - CHROMA);
		yr = (int)(ybot + WHITENESS*(ytop-ybot));
		drawRelCircle(g, xr, yr, 6);
		g.setColor(Color.white);
		drawRelCircle(g, xr, yr, 4);	
		}
	
	private void drawRelLine(Graphics g,int x1, int y1, int x2, int y2) {
		g.drawLine(x1+xcen,y1+ycen,x2+xcen,y2+ycen);
		}
		
	private void drawRelCircle(Graphics g,int xr, int yr, int rad) {
		xr += edgeLen/2;
		yr = edgeLen/2 - yr;
		g.drawOval(xr-rad/2,yr-rad/2,rad,rad);
		}
		
	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
}  
