package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.*;
import java.util.*;
import java.awt.*;  import javax.swing.*;

class Parabola3d { 
int ix;  
int iy;
int iz;
double cgalpha;
double alphamin;
double alphamax;
double linetol;
double width;
double arclen;
double [] b = new double[3];
double [] c = new double[3];
// these are  coefficients to construct a 3-d parabola 
boolean iftalk=false;
	
public double disttoparabola(int x, int y) {
double dist;
    if(ix == 0) 
		dist =  y - parapoint((double )x, 1);
    else
		dist = x - parapoint((double )y, 1);
    if(dist<0)
		dist = (-dist);
    if(dist>.5) dist -=.5;
    return(dist);		  
	}

public double parapoint(double alp, int which) {
double dum;
    alp -= cgalpha;
    if(which == 1)  /* use b[] coefficients */
		return(b[0] + alp*(b[1] +alp*(b[2]))); 
    else // use c[] coefficients
		return(c[0] + alp*(c[1] +alp*(c[2]))); 
	}

/* use this like a constructor. Send it a SwarmOfPoints. If it returns
false then that SwarmOfPoints does not form a valid parfabola. 
If it returns true, then that SwarmOfPoints does form a valid parabola
and this Parabola3d will have the best fit coefficients stored.
*/
public boolean parabolaQ(SwarmOfPoints swarm, int spotthresh, double linetol) {
this.linetol=linetol;
	// Constructor returns null 
int i,nclose;
int maxextent=swarm.getLocations();
int npts= swarm.numPoints;
double cgx, cgy, cgz, curvature, avgdist;
SwarmPoint apoint;
int [] alpha = new int[npts];
int [] beta = new int[npts];
int [] gamma = new int[npts];

cgx=swarm.locations[0][2];
cgy=swarm.locations[1][2];
cgz=swarm.locations[2][2];
if(maxextent == swarm.locations[1][1]-swarm.locations[1][0]) {
    iy=0; /* parameterize by y */
    ix=1;
    iz=2;
    cgalpha = cgy;
    alphamin = swarm.locations[1][0]; 
    alphamax = swarm.locations[1][1]; 
    }
else {
    ix=0;   /* parameterize by x */
    iy=1; 
    iz=2; 
    cgalpha = cgx;
    alphamin = swarm.locations[0][0]; 
    alphamax = swarm.locations[0][1]; 
    }
    
/* 3-D if(maxextent == extents[2][1]-extents[2][0]) {
    parabola->iz=0;  
    parabola->ix=1; 
    parabola->iy=2; 
    parabola->cgalpha = cgz;
    parabola->alphamin = extents[2][0]; 
    parabola->alphamax = extents[2][1]
    }  */

	apoint = swarm.firstPoint; 
	i=0;	
//  Transfer the swarm points into the alpha,beta,gamma arrays. 
//  The parabola will be parameterized by alpha.
    while(apoint != null) {
		triage(apoint.x,  ix, i, alpha, beta, gamma);
		triage(apoint.y,  iy, i, alpha, beta, gamma);
	    /* in 3-D case,...
	    triage(swarm->z,  parabola->iz, i, alpha, beta, gamma); */
		apoint = apoint.neighborPoint;
		++i;
		}
   	
    if(!fitcoeffs(npts,alpha,cgalpha,beta,b))
		return(false);
   /*  for 3-D case:  
   if(!fitcoeffs(npts, alpha, parabola->cgalpha,  gamma, parabola->c))
	return(false);   */

    arclen = arclength();	
    // total arclength along parabola 
    /* 3-D parabola->width = sqrt((double )npts/((parabola->arclen)*3.14159)); */
   	if(arclen >= 1.) {
    	if((width = (double )npts/(2.0*arclen)) < .5)  
    		width = .5;
    	}
    else width = 1.;
   
    if((curvature = b[2])<0) 
    	curvature=(-curvature);
          
    if (iftalk && maxextent > 0) {
    	talk(npts+" pt swarm w/ maxextent "+maxextent+", ix="+ix+", iy="+iy+" has arclen="+arclen+
    			", curvature="+curvature); 
    	talk("width="+width+", coefficients are: b0="+b[0]+",  b1="+b[1]+",  b2="+b[2]);
    	talk("alpha endpoints are: ("+alphamin+","+parapoint(alphamin, 1)+")  and: ("+
    			alphamax+","+parapoint(alphamax, 1)+")");
    	}
    
   if(curvature > 1./(double )spotthresh) {
   	if(iftalk) talk("Failed because curvature="+curvature+" > "+2./(double )spotthresh); 
	return false;
	}
	
    avgdist = avgDistTo(swarm);
   	if(avgdist > linetol*width) {
		if(iftalk) talk("Failed because avgdist="+avgdist+" >  linetol*width="+linetol*width); 
		return(false);
		}
    else return(true);     
	}
	
	private boolean fitcoeffs(int npts, int [] alpha, double alpha0, int [] aray,double [] coef) {
		double a11=0, a12=0, a13=0, a21=0, a22=0, a23=0, a31=0, a32=0, a33=0, r1=0, r2=0, r3=0,det;
		int i, si, sisq;
		double toosmall=1.e-30;

  		for(i=0;i<npts;i++) {
    		a11 += 1;
    		si = (int)(alpha[i]-alpha0);
    		sisq = si*si;
    		a12 += si;
   			a13 += sisq;
    		a23 += si*sisq;
    		a33 += sisq*sisq; 
    		r1 += aray[i];
    		r2 += si*aray[i];
    		r3 += sisq*aray[i];
    		}
    	a21=a12;
    	a22=a13;	    a31=a13;
    	a32=a23;
   
   /* Use cramer's rule */
   		det = det3(a11, a12, a13, a21, a22, a23, a31, a32, a33);
   		if(det < toosmall && det > (-toosmall)) {
       		if(iftalk) talk("fitcoeffs() det="+det);
			return(false); 
			}
   		coef[0] = det3(r1, a12, a13, r2, a22, a23, r3, a32, a33)/det; 
   		coef[1] = det3(a11, r1, a13, a21, r2, a23, a31, r3, a33)/det;
   		coef[2] = det3(a11, a12, r1, a21, a22, r2, a31, a32, r3)/det;
   		return(true);
		}

	private double det3(double a11,double a12,double a13,double a21,double a22,double a23, 
			double a31,double a32,double a33)  {
		double det = a11*(a22*a33 - a32*a23);
   		det -= a12*(a21*a33 - a31*a23);
    	det += a13*(a21*a32 - a22*a31);
    	return(det);
		}
	
public double  avgDistTo(SwarmOfPoints swarm) {
	// return how the av eragedistance between pixels in the parabola and the swarm
	int npts=0, i;
	double avgdist=0;
	SwarmPoint apoint;
	apoint = swarm.firstPoint; 	
    while(apoint != null) {
		avgdist += disttoparabola(apoint.x, apoint.y);
		++npts;
		apoint = apoint.neighborPoint;
		}
	return(avgdist /((double )npts));
	}

public double arclength() {
	int i;
	double d1, d2, d0, p1, p2, pp1, pp2, pp0;
    int npts = (int )(2*(alphamax - alphamin));
    double alp = .5 + alphamin;
    double arclen=0;
    double p0=alphamin;
    p1 = parapoint(p0, 1); 
    /* 3-D p2 = parapoint(parabola, p0, 2); */
    while(alp < alphamax) {
		pp0=alp;
		pp1 = parapoint(pp0, 1); 
		/* 3-D pp2 = parapoint(parabola, pp0, 2); */
		d0=pp0-p0;
		d1 = pp1-p1;
		/* 3-D d2 = pp2-p2; */
		arclen += Math.sqrt(d0*d0 + d1*d1); /* for 3-D case add in d2*d2 */
		p0=pp0;	p1=pp1;	/* 3-D p2 = pp2; */
		alp += .5;
		}
 	return(arclen);          
	}

	private void triage(int xyz,  int ixyz, int i, 
			int [] alpha, int [] beta, int [] gamma) {
    	switch (ixyz) {
		case (0) :
		    alpha[i]=xyz;
	    	break;
		case (1) :
	    	beta[i]=xyz;
	    	break;
		case (2) :
	    	gamma[i]=xyz;
	    	break;
		}	        
	}
	
	private int inbounds(int xy,int NRC) {
		int i=xy;
		if(i<0) i=0;
		if(i>=NRC) i=NRC-1;
		return i;
		}
		
public int paintOnOutRas(short [] outras,int NCOLS,int NROWS, short [] inras)  {
// paints this parabola onto outras and returns the number of pixels turned on.
// It turns on only pixels that were off.
	int i, x, y, z, ylow, yhi, xlow, xhi, xend, yend, dx, dy, offset, j,k;
	double q;
	double distsq = width*width;
	double alp = alphamin;
	int non=0;
	boolean paintIt;
	while(alp <= alphamax) {
    	q = parapoint(alp, 1);
    	if(ix == 0) {
			xlow=inbounds((int )(alp+.49), NCOLS);
			xhi=xlow;
			ylow = inbounds((int )(q + .49  - width), NROWS);
			yhi = inbounds((int )(q + .49  + width), NROWS); 
			}
    	else {
			ylow=inbounds((int )(alp+.49), NROWS);
			yhi =ylow;
			xlow = inbounds((int )(q + .49  - width), NCOLS); 
			xhi = inbounds((int )(q + .49  + width), NCOLS); 
			}
    	for(x=xlow;x<=xhi;x++)
      		for(y=ylow;y<=yhi;y++) {
      			offset=y*NCOLS+x;
      			if(outras[offset] < 128) { // the voxel is currently off, so it makes sense to paint it on
      				paintIt = inras[offset] >= 128 ? true : false;
      				j=x-1; k=y-1;
      				while(!paintIt && k <= y+1) { // see if an adjacent pixel is on
      					if(j>=xlow && j<=xhi && k>=ylow && k<=yhi && inras[k*NCOLS+j] >= 128)
      						paintIt=true; // because the a voxel near this one is on in input raster
      					if(++j > x+1) {
      						j=x-1;
      						++k;
      						}
      					}      				
      				if(paintIt) {
	    				++non;
	    				outras[offset]=255;
	    				}
	   				}
	   			}
    	alp += 0.5;
    	}
    
   // round the ends of the parabola... */
    alp = alphamin;
    i=2;
	while(i>0) {
    q = parapoint(alp, 1);
    if(ix == 0) {
		xend= inbounds((int )(alp + .49),NCOLS);
		yend=inbounds((int )(q + .49), NROWS);
		}
    else {
		yend=inbounds((int )(alp + .49), NROWS);
		xend= inbounds((int )(q + .49), NROWS); 
		}
    ylow = inbounds((int )(yend - width), NROWS);    
    yhi =  inbounds((int )(yend + width), NROWS);    
    xlow = inbounds((int )(xend - width), NROWS);    
    xhi =  inbounds((int )(xend + width), NROWS);    
    for(x=xlow;x<=xhi;x++) {
		dx = x-xend;
		for(y=ylow;y<=yhi;y++) {
	   		dy = y-yend;
	    	if(dx*dx + dy*dy <= distsq) {
      			offset=y*NCOLS+x;
      			if(outras[offset] < 128) {
	    			++non;
	    			outras[offset]=255;
	   				}
				}
	   		}
		}
    --i;
    alp = alphamax; // the other end
    }    
	return (non);
	}
	
static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
}
