package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.*;
import java.util.*;
import java.awt.*;  import javax.swing.*;

class RGBLUTCanvas extends Canvas implements  java.awt.event.MouseListener {
	int [] currentRedLUT, currentGreenLUT, currentBlueLUT;
	int [] newLUT;
	int lastX=0;
	int lastY=0;
	int mouseStartX = 0;
	int mouseStartY = 0;
	CustomRGBLUT parent=null;
	boolean justFinished=true;
	int editColor=0;
	int newRed =255;
	int newGreen = 0;
	int newBlue = 255;
	int oldParentWidth=0,  oldParentHeight=0;
	Color bblue = new Color(0,  0 , 255);
	
	public RGBLUTCanvas(CustomRGBLUT parent){
		int i;
		this.parent=parent;
		currentRedLUT = new int[256];
		currentGreenLUT = new int[256];
		currentBlueLUT = new int[256];
		newLUT = new int[256];
		for(i=0;i<256;i++) {
			currentRedLUT[i]=i;
			currentGreenLUT[i]=i;
			currentBlueLUT[i]=i;
			newLUT[i]=0;
			}
		lastX=0; lastY=0;
		setBackground(new Color(64, 0, 64));
		setSize(256,256);
		justFinished=true;
		addMouseListener(this);
   		}
   	
   	public int [] getRedLUT() {
   		int [] theLUT = new int[256];
   		for(int i=0;i<256;i++)
   			theLUT[i] = currentRedLUT[i];
   		return theLUT;
   		}
   	
   	public int [] getGreenLUT() {
   		int [] theLUT = new int[256];
   		for(int i=0;i<256;i++)
   			theLUT[i] = currentGreenLUT[i];
   		return theLUT;
   		}
   	
   	public int [] getBlueLUT() {
   		int [] theLUT = new int[256];
   		for(int i=0;i<256;i++)
   			theLUT[i] = currentBlueLUT[i];
   		return theLUT;
   		}
	
	public void transferLUT(int src, int dst) {
	    int [] from= currentRedLUT;
	    int []  to= currentRedLUT;
	    switch (src) {
		case 0: from = currentRedLUT;
		break;
		case 1: from = currentGreenLUT;
		break;
		case 2: from = currentBlueLUT;
		break;
		}
	    switch (dst) {
		case 0: to = currentRedLUT;
		break;
		case 1: to = currentGreenLUT;
		break;
		case 2: to = currentBlueLUT;
		break;
		}
	    copyLUT(from, to);
	    justFinished = true;
	    repaint();
	    }
   	
   	public void getNewLUT(int iwh) {
		editColor=iwh;
   		lastX=0;
   		lastY=0;
		justFinished=true;
   		talk("click away...");
		    switch (editColor) {
			case 0:
			    copyLUT(currentRedLUT, newLUT);
			    newRed=128; newGreen= 0; newBlue =  0;
			    break;
			case 1:
			    copyLUT(currentGreenLUT, newLUT);
			    newRed=0; newGreen= 128; newBlue =  0;
			    break;
			case 2:
			    copyLUT(currentBlueLUT, newLUT);
			    newRed=0; newGreen= 0; newBlue =  128;
			    break;
			}
		repaint();
   		}

	private void drawLUT(GraphicsPs gg,int [] LUT,int kk) {
		int i;
		int x,y,xn=0, yn=255-LUT[0];
		for(i=0;i<=kk;i++) {
			x=xn;	y=yn; 
			xn=i;	yn=255-LUT[i];
			gg.drawLine(x,y,xn,yn);
			}
		}		

	private void copyLUT(int [] L,  int [] M) {
	    for(int i=0;i<256;i++)
		M[i]=L[i];
	    }
	
	public void mouseClicked(java.awt.event.MouseEvent event) {
		// talk("Mouse Clicked at ("+event.getX()+", "+event.getY()+")");
		}
		
	public void mousePressed(java.awt.event.MouseEvent event) {
		    mouseStartX = event.getX();
		    mouseStartY = event.getY();
		    if(mouseStartX>255) mouseStartX=255;
		    if (mouseStartX<0) mouseStartX=0;
		    if(mouseStartY>255) mouseStartY=255;
		    if (mouseStartY<0) mouseStartY=0;
		}
		
		
	public void mouseEntered(java.awt.event.MouseEvent event) {
		// talk("Mouse entered at ("+event.getX()+", "+event.getY()+")");
		}
		
	public void mouseExited(java.awt.event.MouseEvent event) {
		// talk("Mouse exited at ("+event.getX()+", "+event.getY()+")");
		}
		
	public void mouseReleased(java.awt.event.MouseEvent event) {	
		int x = mouseStartX;
		int y = 255-mouseStartY;
		if(x>lastX) {
			for(int i=lastX; i<=x;i++) 
				newLUT[i] = lastY + ((i-lastX)*(y-lastY))/(x-lastX);
			lastY=y;
			lastX=x;
		  if(255-lastX <= 3) {
			for(int i = lastX+1;i<256;i++) newLUT[i]=lastY;
			switch (editColor) {
			    case 0:
				copyLUT(newLUT,currentRedLUT);
				break;
			    case 1:
				copyLUT(newLUT,currentGreenLUT);
				break;
			    case 2:
				copyLUT(newLUT,currentBlueLUT);
				break;
			    }
			lastX=255;
			justFinished=true;
			talk("OK the LUT is complete...");
			}
		   }
		repaint();
		}

	public void paint(Graphics g) {
		//talk("This should never happen. RGBLUTCanvas.paint() called");
		paint2D((Graphics2D)g);
		}
		
	public void update(Graphics g) {
		// talk("RGBLUTCanvas.update() called. Should never happen");
		paint2D((Graphics2D)g);
		}

	public void update2D(Graphics2D g) {
		paint2D(g);
		}

	public void paint2D(Graphics2D g,  boolean ifVirgin) {
	    justFinished=ifVirgin;
	    talk("virgin paint");
	    repaint();
	    }
	    
	public void paint2D(Graphics2D g) {
		GraphicsPs gg = new GraphicsPs(parent.width,parent.height,g,
			parent.ifWritePostscriptFile);
	    if(justFinished || parent.width != oldParentWidth || 
		parent.height != oldParentHeight || parent.ifWritePostscriptFile) {
		parent.ifWritePostscriptFile = false;
		int grid;
		oldParentWidth = parent.width; 
		oldParentHeight = parent.height;
		g.setColor(Color.darkGray);
		g.fillRect(0,0,255,255);
		gg.newColor(128,128,128);
		gg.drawLine(0,255,255,0); // the 45o line
		// draw the grid lines...
		for(grid=255; grid >= 0 ;grid -= 32) { 
		    gg.drawLine(grid, 0, grid, 255);
		    gg.drawLine(0, grid, 255, grid);
		    }
		gg.newColor(0,0,255);
		drawLUT(gg,currentBlueLUT,255);
		gg.newColor(255,0,0);
		drawLUT(gg,currentRedLUT,255);
		gg.newColor(0,255,0);
		drawLUT(gg,currentGreenLUT,255);
		justFinished=false;
		talk("redraw everything");
		}
		parent.ifWritePostscriptFile = false;
	    if(lastX<255) { // draw the new forming LUT
			gg.newColor(newRed,newGreen,newBlue);
			drawLUT(gg,newLUT,lastX);
			}
	gg.finish();
	}
	
	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
}  
