package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;  import javax.swing.*;
import java.awt.image.*;
import java.util.*;
import java.lang.*;
import java.io.File;

public class RGBMarker {
boolean ifSelected;
ShowPicJPanel parent=null;
BioradCanvas curStack;
MarkerLeaf [] leaf = {new MarkerLeaf(this),new MarkerLeaf(this),
	    new MarkerLeaf(this), };
double [] cg = {0., 0., 0.};
double wide = 50.;
double high = 50.;
boolean ifVisible=false;

    RGBMarker(ShowPicJPanel parent) {
	this.parent=parent;
	ifSelected = false;
	// make the three leaves; each has a color complementary to the 
	// color of the channel it marks..
	leaf[0].setProps(new Color(0, 255, 255),  0);
		// cyan leaf[0] marks red channel 
	leaf[1].setProps(new Color(255, 0, 255),  1);
		// magenta leaf[1] marks green channel 
	leaf[2].setProps(new Color(255, 255, 0),  2);
		// yellow leaf[2] marks red channel 
	setStats(30., 30.);
	cg[0] = 0.;
	cg[1] = 0.;
	cg[2] = 0.;
	}

    public void adoptTheseLeafShapes(int redshape, 
	    int greenshape, int blueshape) {
	leaf[0].myShape=redshape;
	leaf[1].myShape=greenshape;
	leaf[2].myShape=blueshape;
	}
	    
    public boolean isActive() { // return trtue if not in the dock where I'm in 
				// if both my horizontal and vertical coordinates are <5the dock 
	return(cg[0]>4 || cg[1]>4);
	}
	
    public void adoptNewStack(BioradCanvas curStack) {
	this.curStack=curStack;
	for(int i=0;i<3;i++)
	    leaf[i].curStack = curStack;
	stayInBox();
	equilibrate();
	}
	
	public double [] tellAbsLoc() {
		double [] xyz = {cg[0],cg[1],cg[2]};
		return xyz;
		}
	
	public void returnToDock() {
		coincide();
		for(int i=0;i<3;i++)
	    	cg[i]=0;
		}
	
	public double [] tellLeafOffset(int chan) {
		return(leaf[chan].tellOffsets());
		}
	
	public double [] tellAbsLoc(int chan) { // return abs location of
			//   leaf[chan]
		return (leaf[chan].tellAbsLoc());
		}
	
	public void archiveMarks(ArchiveMarks disker,boolean ifWrite,int id) {
		int i,k;
		double [] leafoffs={0,0,0};
		double badval = 7777.777;
		if(ifWrite) { // write mark info to disk
			for(i=0;i<3;i++) 
				disker.writeNum(cg[i],(" = cg["+i+"] of mark "+id));
			disker.writeNum(wide,(" = half width of  mark "+id));
			disker.writeNum(high,(" = half height of  mark "+id));
			for(k=0;k<3;k++) {
				leafoffs=leaf[k].tellOffsets();
				disker.writeNum(leaf[k].myShape,(" = leaf shape for chan "+k+
							"  for marker "+id));
				for(i=0;i<3;i++)
					disker.writeNum(leafoffs[i],(" = offset["+i+"] of chan "+k+
							"  for marker "+id));
				}
			return;
			}
		else { // recover art color info from disk and set text fields 
			for(i=0;i<3;i++) 
				cg[i]=disker.getNextFileDouble(badval);
			setStats(disker.getNextFileDouble(badval), 
				disker.getNextFileDouble(badval));
 			for(k=0;k<3;k++) {
				leaf[k].myShape = (int)disker.getNextFileDouble(badval);
				for(i=0;i<3;i++)
					leaf[k].myOffset[i]= disker.getNextFileDouble(badval);
				}
			stayInBox();
			equilibrate();
		}
	}

    public double absDistanceFrom(double [] xyz) {
	double ddst = cg[0]-xyz[0];
	double dist = ddst*ddst;
	ddst = cg[1]-xyz[1];
	dist += ddst*ddst;
	ddst = cg[2]-xyz[2];
	dist += ddst*ddst;
	return(Math.sqrt(dist));
	}
	    
    public double  distanceToMouse(int mx, int my) {
		// returns the distance of this marker to the mouse
		double [] mxyz = {(double)mx, (double)my, (int)(curStack.curJFrame)};
		curStack.unScaleCoords(mxyz);
		return (absDistanceFrom(mxyz));
		}
		
    public void  moveTo(double [] xyz) {
    	// scaled coordinates come in. 
	for(int i=0;i<3;i++)
	    cg[i]=xyz[i];
    curStack.unScaleCoords(cg);
	stayInBox();
	equilibrate();
	}
	
    public void  moveToMouseLoc(int mx, int my) {
    	// scaled coordinates come in. 
	    cg[0]=(double)mx;
	    cg[1]=(double)my;
	    cg[2] = curStack.curJFrame;
	    talk("before...RGBMarker.moveToMouseLoc, Z=curJFrame="+cg[2]);
   	    curStack.unScaleCoords(cg);
	    talk("after unScaleCoords...RGBMarker.moveToMouseLoc, Z=curJFrame="+cg[2]);
	    stayInBox();
	    talk("after stayInBox...RGBMarker.moveToMouseLoc, Z=curJFrame="+cg[2]);
	    equilibrate();
	}
	
    public void  moveLeafToMouseLoc(int chan, int mx, int my) {
	if(chan<3 && chan>=0)
	    leaf[chan].moveOffsetsToHitMouseLoc(mx, my);
	equilibrate();
	}
	
    public void moveXYTo(double x,  double y) {
	cg[0]=x;
	cg[1]=y;
    curStack.unScaleCoords(cg);// leaves z-coordinate untouched;
	stayInBox();
	equilibrate();
	}

    public void chanOffsetMove(int chan, double dx,  double dy,  double dz) {
    // the move increments are in absolute coordinates.
	if(chan<3 && chan >=0)
	    leaf[chan].moveRel(dx, dy, dz);
	equilibrate();
	}

    public void equilibrate() { // relocate cg[] to center of gravity of the
	    // three leaf locations...
	double [] redloc = leaf[0].tellAbsLoc();
	double [] greenloc = leaf[1].tellAbsLoc();
	double [] blueloc = leaf[2].tellAbsLoc();
	double [] newcg = {(redloc[0]+greenloc[0]+blueloc[0])/3., 
	    (redloc[1]+greenloc[1]+blueloc[1])/3., 
		(redloc[2]+greenloc[2]+blueloc[2])/3.};
	double dx = cg[0] - newcg[0];
	double dy = cg[1] - newcg[1];
	double dz = cg[2] - newcg[2];
	cg[0] = newcg[0];   cg[1] = newcg[1];   cg[2] = newcg[2];   
	for(int i=0;i<3;i++) 
	    leaf[i].moveRel(dx, dy, dz);
	}
		
    public void select(boolean ifsel) {
	ifSelected=ifsel;
	for(int i=0;i<3;i++) 
	    leaf[i].select(ifSelected);
	}
	
    public void appear() {
	ifVisible=true;
	for(int i=0;i<3;i++) leaf[i].ifVisible=true;
	}

    public void disappear() {
	ifVisible=false;
	for(int i=0;i<3;i++) leaf[i].ifVisible=false;
	}

    public void toggleVisibilty() {
	ifVisible= !ifVisible;
	for(int i=0;i<3;i++) leaf[i].ifVisible=ifVisible;
	}

    public void setStats(double wide, double high) {
	this.wide=wide; // absolute sizes used...
	this.high=high;
	dictateLeafStats();
	}
	
    private void dictateLeafStats() {
	for(int i=0;i<3;i++) {
	    leaf[i].wide = wide;
	    leaf[i].high = high;
	    }
	//talk("marker is "+wide+" wide and "+high+" high");
	}
	
    public void boostWidth(){
	wide += 1.;    
	dictateLeafStats();
	}

     public void boostHeight(){
	high += 1.;    
	dictateLeafStats();
	}

    public void cutWidth(){
	if((wide -= 1.) <1.) wide = 1.;    
	dictateLeafStats();
	}

     public void cutHeight(){
	if((high -= 1.) <1.) high = 1.;    
	dictateLeafStats();
	}

 
    public void grow() {
	grow(1);
	}
	
    public void grow(int rup) {
	double aspr = high/wide;
	wide += (double)rup;
	high += (aspr*rup);
	dictateLeafStats();
	}

    public void shrink() {
	shrink(1);
	}
    public void shrink(int rdn) {
	double aspr = high/wide;
	if((wide -= (double)rdn) <1.)
	    wide=1.0;
	if((high -= (aspr*rdn)) < 1.) 
	    high =1.0;
	dictateLeafStats();
	}

    public void stayInBox() {
	for(int i=0;i<3;i++)
	    if(cg[i]<0.) cg[i] =0;;
	if(cg[0]>parent.fullWidth-1) cg[0] = parent.fullWidth-1;
	if(cg[1]>parent.fullHeight-1) cg[0] = parent.fullHeight-1;
	if(cg[2] >= (curStack.NJFrames-1))
	     cg[2] = (curStack.NJFrames-1);
	for(int i=0; i<3;i++)
	    leaf[i].stayInBox();
	}

    public void coincide() {
	for(int i=0;i<3;i++)
	    leaf[i].nullOffsets();
	stayInBox();    
	}

   public void draw(Graphics g, int iwh,  int zsect, int xorg, int yorg) {
   // iwh==0 means draw if you are visible and your z is close enough to zsect.
   // iwh==1 means draw (visible of not) if your z is close enough to zsect.
   // iwh==2 means unconditional draw.
	for(int i=0; i<3;i++)
	    leaf[i].draw(g, iwh, zsect,  xorg,  yorg);
	}

    static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
		
    public String myStatsString() {
	String stg = "located at ("+(int)cg[0]+", "+(int)cg[1]+
		", "+(int)cg[2]+")  width="+wide+",  height="+high;
	return (stg);
	}

}
