package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;  import javax.swing.*;
import java.awt.image.*;
import java.util.*;
import java.lang.*;
import java.io.*;

class SegPic {
	SegTowerCanvas parent=null;
	//  parent.parent is segTower
	//  parent.parent.parent is segmenter, the main data segmentation panel which spawns all the
	//  segmenter nodes, and which contains the lists of colors called maskrgbs[][]
	int [] wholePic;
	int npaths=1,nsteps=1;
	int wholeRows=-1,wholeCols=-1,npix=-1;
	int stampXorg=-1, stampYorg=-1, stampWidth=-1,  stampHeight=-1;
	boolean isStampCurrent = false;
	static String [] names = {"empty","new","","contender","bestYet"};
	int status = -1; // 0 means just created, no button pushed
					 // 1 means some click was made in this pic stamp
					 // 2 means contender
					 // 3 means best
					 // -1 means empty
	int alpha =(255 << 24);
	
	int [][][] allNodePars;
	int numPaths=1;
	int numSteps=8;	
	int nPixOn=0;	
	int sectionNumber=0;
	
	public int [] getCopyOfWholePic() {
		if(wholePic==null)
				return null;
		int npixes = wholePic.length;
		int [] retn = new int[npixes];
		for(int i=0; i<npixes;i++)
			retn[i] = wholePic[i];
		return retn;
		}
	
	public int getGlow(int x, int y) {
		if(x<0 || x>=wholeCols || y<0 || y>= wholeRows)
			return -1;
		int offset = y*wholeCols +x;
		int pix = wholePic[offset];
		int pred = (pix & (255<<16)) >> 16;
		int pgreen = (pix & (255<<8)) >> 8;
		int pblue = pix & 255;
		if((pix=pred) < pgreen) pred=pgreen;
		if(pred < pblue) pred=pblue;
		return pred;
		}
		
	public int getNbhdGlow(int x, int y) {
		if(x<0 || x>=wholeCols || y<0 || y>= wholeRows)
			return -1;
		int yi,xj,i,j,pix = 0,npx=0;
		
		for(i=-2; i<3; i++)
			if((yi=y+i) >= 0 && yi < wholeRows)
				for(j=-2;j<3;j++)
					if((xj=x+j) >= 0 && xj<wholeCols) {
						++npx;
						pix += getGlow(xj,yi);
						}
		
		return (pix/npx);
		}
		
	public void changeStampSizes(int stampWidth,int stampHeight) {
		this.stampHeight=stampHeight;
		this.stampWidth=stampWidth;
		isStampCurrent = false;
		}
		
	public void sepuku() {
		parent=null;
		wholePic=null;
		allNodePars=null;
		// talk("SegPic.sepuku() =================");
		}
		
	public String statusString() {
		return names[status+1];
		}
		
	public SegPic(SegTowerCanvas parent) {
		this.parent = parent;
		}
	
	public boolean attentionPaid() {
		if(status==0) {// 0 means just created, no button pushed
			status=1; // 1 means some click was made in this pic stamp
			isStampCurrent = false;
			parent.repaint();
			return true;
			}
		return false;
		}
	
	public void dismissPic() {
		if(status==3) return;
		empty();
		parent.repaint();
		}
	
	public void promoteToContender() {
		if(status == 2) {
			// it's already a contender; demote it (this button is a toggle)
			talk("demoting contender to ordinary status");
			status=1; 
			isStampCurrent = false;
			parent.repaint();
			return;
			}
		if(status>=0) {
			status=2; // 2 means contender
			isStampCurrent = false;
			parent.repaint();
			}
		return;
		}
	
	public void demoteContender() {
		if(status == 2) {
			// it's already a contender; demote it (this button is a toggle)
			talk("demoting contender to ordinary status");
			status=1; 
			isStampCurrent = false;
			parent.repaint();
			return;
			}
		return;
		}
	
	public void revokeBestStatus() {
		if(status == 3) {
			status=2;
			isStampCurrent = false;
			}
		}
		
	public void promoteToBestYet() {
		if(status>=0) {
			status=3; // 2 means contender
			isStampCurrent = false;
			parent.parent.parent.rebuildWholeChain(numPaths, numSteps, allNodePars);
			parent.repaint();
			}
		}
	
	private void saveAllNodePars(int np, int ns, int [][][] prs) {
		if(np != numPaths || ns != numSteps || allNodePars==null) {
			numPaths=np;
			numSteps=ns;
			allNodePars = new int[numPaths][numSteps][12];
			}
		for(int i=0;i<np;i++)
			for(int j=0;j<ns;j++) 
				for(int k=0;k<12;k++)
					allNodePars[i][j][k] = prs[i][j][k];
		}

	public int colorDataThroughMask(SegmenterNode showNd, int redmark, int greenmark, int bluemark, 
				int reddim, int greendim, int bluedim) {
			nPixOn=0;
			short onlev=(short)127;
			int i,raw,red,green,blue;
				
			int inp2=showNd.inPath2;
			int ins2= showNd.inStep2;
			short [] indat=null;
			short [] mask = showNd.shortRas;
				 
			if(inp2<0 || ins2<0)  
				switch (showNd.whatAmI) {
					case 22 : // mask 1
					indat = (parent.parent.parent.nodes[0][1]).shortRas;
					break;
					case 23 : // mask 2
					indat = (parent.parent.parent.nodes[0][2]).shortRas;
					break;
						
					case 24 : // mask 3
					indat = (parent.parent.parent.nodes[0][3]).shortRas;
					break;
						
					case 25 : // mask 4
					indat = (parent.parent.parent.nodes[0][1]).shortRas;
					break;
						
					default :
					talk("Catastrophic switch choice???");
					return 0;
						
					}
				
				else 
					indat=showNd.parent.nodes[inp2][ins2].shortRas;
					
			for(i=0;i<npix;i++) {
					raw = indat[i]&255;
				if(mask[i] < onlev) {
					red=(raw*reddim)/255;;
					green=(raw*greendim)/255;
					blue= (raw*bluedim)/255;
					}
				else {
					++nPixOn;
					red = (raw*redmark)/255;
					green = (raw*greenmark)/255;
					blue = (raw*bluemark)/255;
					}
				wholePic[i]= (alpha | (red << 16) | (green << 8) | (blue));
				}
			return nPixOn;
			}

	public void acceptPic(SegmenterNode showNd, int [][][] allNodePars, int numPaths, int numSteps) {
		int whCols = showNd.rasCols;
		int whRows = showNd.rasRows;
		short [] whPic = showNd.shortRas;
		short onlev=(short)127;
		// wholePic[i] = (alpha | (red[i] << 16) | (green[i] << 8) | (blue[i]));
		if(allNodePars != null)
			saveAllNodePars(numPaths,numSteps,allNodePars);
		
		sectionNumber = parent.parent.parent.curSect;
		if(whCols<=0 || whRows <= 0) {
			talk("segPic.acceptPic() got negative nuber of rows or cols and returns without doing anything.");
			return;
			}
		short inshort;
		if(whRows != wholeRows || wholeCols != whCols) {
			wholeRows=whRows;
			wholeCols=whCols;
			talk("SegPic makes new whole pic with "+wholeCols+" cols and "+wholeRows+" rows");
			npix = wholeCols*wholeRows;
			wholePic = new int[npix];
			}
		nPixOn=0;
		if(showNd.whatAmI < 22 || showNd.whatAmI >24 || showNd.MaskColorsData<=0) { // just show the input picture
			for(int i=0; i<npix;i++) {// store the whole image
				inshort = whPic[i];
				if(inshort>onlev) ++nPixOn;
				wholePic[i]= (alpha | (inshort << 16) | (inshort << 8) | (inshort));
				}
			} 
			
		else  if(showNd.MaskColorsData == 1) {// to get here whatAmI == 22, 23, or 24 and MaskColorsData== 1.
				// use the input picture as a mask to look at the raw data through but also show all raw data
				//  parent.parent.parent is segmenter, the main data segmentation panel which spawns all the
				//  segmenter nodes, and which contains the lists of colors called maskrgbs[][]
			int [] mrgb=null; // mrgb will be an array of six colors... 
			if(showNd.whatAmI==22)  //  mask 1                       
				mrgb = parent.parent.parent.maskrgbs[0];				
			if(showNd.whatAmI==23)  //  mask 2                 	
				mrgb = parent.parent.parent.maskrgbs[1];
			if(showNd.whatAmI==24)  //  mask 3     
				mrgb = parent.parent.parent.maskrgbs[2];
				nPixOn=colorDataThroughMask(showNd, mrgb[0],mrgb[1],mrgb[2],mrgb[3],mrgb[4],mrgb[5]);
				}
		else  if(showNd.MaskColorsData == 2) {// to get here whatAmI == 22, 23, or 24 and MaskColorsData == 2.
				// use the input picture as a mask to look at the raw data through
			if(showNd.whatAmI==22)  // red mask          
				nPixOn=showMaskedData(showNd,true);
			if(showNd.whatAmI==23)  // green mask         	
				nPixOn=showMaskedData(showNd,true);
			if(showNd.whatAmI==24)  // blue mask                     
				nPixOn=showMaskedData(showNd,true);
			}
		else  if(showNd.MaskColorsData == 3) {// to get here whatAmI == 22, 23, or 24 and MaskColorsData == 3.
				// just show the raw data
			if(showNd.whatAmI==22)  // red mask          
				nPixOn=showMaskedData(showNd,false);
			if(showNd.whatAmI==23)  // green mask         	
				nPixOn=showMaskedData(showNd,false);
			if(showNd.whatAmI==24)  // blue mask                     
				nPixOn=showMaskedData(showNd,false);
			}
		status = 0;
		isStampCurrent=false;
		parent.parent.showYourself();
		}
		
	public  byte [] getMaskedDataForOneChannel(SegmenterNode showNd, double nonMaskFrac) {
		short onlev=(short)127;
		short [] indat=null;
		short [] mask = showNd.shortRas;
		short nonMaskFac = (short)(nonMaskFrac*255);
		
		if(mask==null)
			return null;
		switch (showNd.whatAmI) {
			case 22 : // red chan1
				indat = (parent.parent.parent.nodes[0][1]).shortRas;
				System.out.println("SegPic.getMaskedDataForOneChannel( red )");
				break;
			case 23 : // green chan2
				indat = (parent.parent.parent.nodes[0][2]).shortRas;
				System.out.println("SegPic.getMaskedDataForOneChannel(    green )");
				break;		
			case 24 : // blue chan3
			indat = (parent.parent.parent.nodes[0][3]).shortRas;
			System.out.println("SegPic.getMaskedDataForOneChannel(         blue )");
						break;
		
			default :
					return null;
			}
		
		if(indat==null)
				return null;
		npix=ShowPicJPanel.fullHeight*ShowPicJPanel.fullWidth;
		byte [] retn = new byte[npix];
		for(int i=0;i<npix;i++) {
			retn[i] = (byte)indat[i];
			if(mask[i] < onlev)
				retn[i] = (byte)((nonMaskFac*indat[i])/255);
			}
		return retn;
		}
	
	public int showMaskedData(SegmenterNode showNd, boolean ifUseMask) {
			int i,raw,red,green,blue;
			nPixOn=0;
			short onlev=(short)127;
				int inp2=showNd.inPath2;
				int ins2= showNd.inStep2;
				short [] indat=null;
				short [] mask = showNd.shortRas;
				 
				if(inp2<0 || ins2<0)  
					switch (showNd.whatAmI) {
						case 22 : // red chan1
						indat = (parent.parent.parent.nodes[0][1]).shortRas;
						break;
						case 23 : // green chan2
						indat = (parent.parent.parent.nodes[0][2]).shortRas;
						break;
						
						case 24 : // blue chan3
						indat = (parent.parent.parent.nodes[0][3]).shortRas;
						break;
						
						case 25 : // red chan1
						indat = (parent.parent.parent.nodes[0][1]).shortRas;
						break;
						
						default :
							talk("Catastrophic switch choice???");
							return 0;
							
						}
				
				else 
					indat=showNd.parent.nodes[inp2][ins2].shortRas;
					
		if(ifUseMask) {
			  for(i=0;i<npix;i++) {
				raw = (indat[i] & 255);
				if(mask[i] < onlev) 
					raw=0;
				else 
					++nPixOn;					
				wholePic[i]= (alpha | (raw << 16) | (raw << 8) | (raw));
				}
			   }
		else {
			  for(i=0;i<npix;i++) {
				raw = (indat[i] & 255);
				if(raw > onlev) 
					++nPixOn;					
				wholePic[i]= (alpha | (raw << 16) | (raw << 8) | (raw));
				}
			}
			return nPixOn;
			}
					
	public void empty() {
		status = -1;
		isStampCurrent=false;
		}		
	
	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
		
	public boolean  copyStampToTowerRas(int stampXorg, int stampYorg, int stampWidth, int stampHeight,
			int towerWidth, int towerRow, int towerBorder, int [] towerRas) {
		if(towerRas==null || wholePic==null ) return false;
		if(this.stampXorg==stampXorg && this.stampYorg==stampYorg && this.stampWidth==stampWidth &&
					this.stampHeight==stampHeight && isStampCurrent) return false;
		this.stampXorg=stampXorg;
		this.stampYorg=stampYorg;
		this.stampWidth=stampWidth;
		this.stampHeight=stampHeight;
		
		int row,col,stampRowOff,towerRowOff,bigYOff;
		int emptyColor = alpha | ((parent.borderRed)/3)<<16  | ((parent.borderGreen)/3)<<8  | ((parent.borderBlue)/3);
		boolean ifInBounds;
		if(status == -1) { // it's empty		  
		  for(row=0;row<stampHeight;row++) { // blank out the towerRaster
			towerRowOff = towerWidth*(towerRow + row) + towerBorder;
			for(col=0;col<stampWidth;col++) {
				towerRas[towerRowOff] = emptyColor;
				++towerRowOff;
				}
			}
			}			
		else {// it's an actual picture (but that picture may not be as big as the current best pic
		  for(row=0;row<stampHeight;row++) { // copy stamp patch of wholePic back into the towerRaster
			bigYOff = row + stampYorg;
			ifInBounds = (bigYOff < wholeRows);
			stampRowOff = stampXorg + wholeCols*(bigYOff);
			towerRowOff = towerWidth*(towerRow + row) + towerBorder;
			for(col=0;col<stampWidth;col++)  {
				towerRas[towerRowOff] = (ifInBounds && (col + stampXorg < wholeCols))? wholePic[stampRowOff] : emptyColor;
				++towerRowOff;
				++stampRowOff;
				}
			}}
		isStampCurrent=true;
		return true; // true return means we made a change in the towerRaster;
		}			

}