package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.*;  import javax.swing.*;

class StackGradient extends JFrame implements java.awt.event.WindowListener {
	int nomwidth=490, nomheight=110;
	JTextField [][] boostFacsText = new JTextField[3][4];
	JTextField [] boostSectText = new JTextField [4];
	JTextField nowAtSectText;
	JTextField [] chanBoostsText =  new JTextField [3];
	int [] boostSect = new int[4];
	double [][] theBoosts = new double[3][4];
	double [] boosts = new double[3];

    public StackGradient() {   
    super("Stack Brightness four-segment Gradient");
    getContentPane().setLayout(new GridLayout(4,6)); 
    setSize(nomwidth, nomheight);
	int sect,chan;
	WidgetMaker wmk = new WidgetMaker(this);
	
	nowAtSectText = wmk.makeAJTextField("",12);
	getContentPane().add(nowAtSectText);
	getContentPane().add(wmk.makeAJLabel("    at section->"));
	for(sect=0;sect<4;sect++) {
		boostSect[sect]=10*sect;
		boostSectText[sect] = wmk.makeAJTextField(""+boostSect[sect],12);
		getContentPane().add(boostSectText[sect]);
		}
		
	for(chan=0;chan<3;chan++) {
		chanBoostsText[chan] = wmk.makeAJTextField("",12);
		getContentPane().add(chanBoostsText[chan]);
		if(chan==0) getContentPane().add(wmk.makeAJLabel("  redChanBoost->"));
		else if(chan==1) getContentPane().add(wmk.makeAJLabel("greenChanBoost->"));
		else if(chan==2) getContentPane().add(wmk.makeAJLabel(" blueChanBoost->"));
		for(sect=0;sect<4;sect++) {
			theBoosts[chan][sect]=1.000;
			boostFacsText[chan][sect] = wmk.makeAJTextField(""+theBoosts[chan][sect],12);
			getContentPane().add(boostFacsText[chan][sect]);
			}
		}
		
	addWindowListener(this);
	wmk=null; // we're done with it
  	}

	public double [] getBoosts(int sectionNum) {
		readTextFields();
		int i,s1=-1,s2=-1,chan;
		double frac;
		if(sectionNum<=boostSect[0]) {
			for(chan=0;chan<3;chan++)
				boosts[chan]=theBoosts[chan][0];
			tellBoosts(sectionNum);
			return boosts;
			}
			
		if(sectionNum>=boostSect[3]) {
			for(chan=0;chan<3;chan++)
				boosts[chan]=theBoosts[chan][3];
			tellBoosts(sectionNum);
			return boosts;
			}
			
		// locate sectionNum in boostSect[s1] <= sectionNum <= boostSect[s2]
		for(i=0;i<4;i++) 
			if(boostSect[i] <= sectionNum)  
					s1=i;
	
		for(i=3;i>=0;i--) 
			if(boostSect[i] >= sectionNum)  
					s2=i;
		if((frac=(double)(boostSect[s2]-boostSect[s1])) > 0)
			frac = (double)(sectionNum-boostSect[s1])/frac;
		for(chan=0;chan<3;chan++)
			boosts[chan] = theBoosts[chan][s1] + frac*(theBoosts[chan][s2] - theBoosts[chan][s1]);
		tellBoosts(sectionNum);
		return boosts;
		}

	private void tellBoosts(int sectNum) {
		nowAtSectText.setText("at sect "+sectNum);
		int intpart;
		int fracpart;
		String boostg;
		for(int chan=0;chan<3;chan++) {
			intpart = (int)boosts[chan];
			fracpart = (int)(1000*(boosts[chan]-intpart));
			if(fracpart>=100) // all this to prevent printing too many significant digits
				boostg = ""+intpart + "." + fracpart;
			else if(fracpart>=10)
				boostg = ""+intpart + ".0" + fracpart;
			else 
				boostg = ""+intpart + ".00" + fracpart;
			switch (chan) {
				case 0:
				default:
					chanBoostsText[chan].setText(("Rboost="+boostg));
					break;
				case 1:
					chanBoostsText[chan].setText(("Gboost="+boostg));
					break;
				case 2:
					chanBoostsText[chan].setText(("Bboost="+boostg));
					break;
					}
			}
		}
			
	private void readTextFields() {
		int i,j,k,chan,sect;
		double bst;
		for(sect=0;sect<4;sect++) {
			boostSect[sect] = (int)(Double.valueOf(boostSectText[sect].getText()).doubleValue());
			for(chan=0;chan<3;chan++) 
				theBoosts[chan][sect] = Double.valueOf(boostFacsText[chan][sect].getText()).doubleValue();
			}
		// it the section numbers are not in ascending order, re-arrange that.
		if(boostSect[0] <= boostSect[1] && boostSect[1] <= boostSect[2] && boostSect[2] <= boostSect[3])
				return;
		// the sections at which boosts are specified are out of order. Put them in order.
		for(i=0;i<3;i++) { // bubble sort
			// find the min sect num remaining
			sect=boostSect[i];
			k=i;
			for(j=i+1;j<4;j++)
				if(sect > boostSect[j]) {
					k=j;
					sect = boostSect[j];
					}
				if(k != i) { // swap columns k and i
					// already sect == boostSect[k];
					boostSect[k] = boostSect[i];
					boostSect[i] = sect;
					for(chan=0;chan<3;chan++) {
						bst = theBoosts[chan][k];
						theBoosts[chan][k] = theBoosts[chan][i];
						theBoosts[chan][i] = bst;
						}
					}
			}
		// now reset all the text fields...
		for(sect=0; sect<4; sect++) {
			boostSectText[sect].setText(""+boostSect[sect]);
			for(chan=0;chan<3;chan++)
				boostFacsText[chan][sect].setText(""+theBoosts[chan][sect]);
			}
		}
	
// catch WindowListener events  
	public void windowActivated(java.awt.event.WindowEvent event) {
		// talk("window activated");
		}
	public void windowDeactivated(java.awt.event.WindowEvent event) {
		//  talk("window Deactivated");
		}
	public void windowOpened(java.awt.event.WindowEvent event) {}
	public void windowIconified(java.awt.event.WindowEvent event) {}
	public void windowDeiconified(java.awt.event.WindowEvent event) {}
	public void windowClosing(java.awt.event.WindowEvent event) {
	    setVisible(false);
		}
	public void windowClosed(java.awt.event.WindowEvent event) {
		}	 

// catch button and menu events...

	public void changeJPanelSize(double ratio) {
		int pwidth = getSize().width;
		int pheight = getSize().height;
		pwidth = (int)(ratio*pwidth +.4999);
		pheight = (int)(ratio*pheight +.4999);
		setSize(pwidth,pheight);
		show();
		}

	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}
}

