package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.*;  import javax.swing.*;
import java.awt.event.*;

class Stats extends JFrame implements java.awt.event.ActionListener,java.awt.event.ItemListener,java.awt.event.WindowListener {
	ShowPicJPanel picJPanel;
	// picJPanel.curStack is the BioradCanvas current stack.... Fredraw
	boolean ifReallyMakeHistogram = true; // set to false when Stats reads in a previously stored
	// histogram file. Pushing this panel's "make the histogram" button always sets ifReallyMakeHistogram=true;
	// the point is to let the user actually trigger making core profiles that may take a while.
	JButton globalLocalTog,pinFreeCoreBoxToggle;
	JPanel panB;
	int cylinderWarningCount=0;
	
	int nomwidth=330, nomheight=210;
	JTextField threshRed,threshGreen,threshBlue, zboxtop, zboxbot,
		scaleTotalPerBoxPix,areaPixels,transectPixels,transectWidth,
			transectBlur,pixPerMicron;
	int zmin=0,zmax=10;
	int Rthresh,Gthresh,Bthresh;
	/* On october 1998 I changed Rthresh (threshRed) to denote the percentage of 
	dimmest pixels to omit. That is, Rthresh=15 means omit the 15% of pixels which are dimmest.
	Likewise for green and blue... */
	int tranwidth=30,tranblur=4;
	int colorChoice=1; // 1->red only, 2-->green only, 3-->blue only, 4-->all 3 RGB
	int histogramType=1; // 1 is histogram in region of this section
						// 2 means stacked total accumulation histograms in 3-D volume
						// 3 means conventional histogram for all voxels in a 3-D volume
						// 4 means profile along a linear transect in this single section
						// 5 means profile along a linear transect in a 3-D stack.
						// 6 Zcore -discard pix dimmer than RGBdim
						// 7 means do full stats on a directory full of callibration standards pics.
						// 8 Zcore -discard RGBdim % of dimest pixels
						// 9 BleachCorrection -discard pix dimmer than RGBdim
                        // 10 BleachCorrection -discard RGBdim % of dimest pixels
	
	int mouseTask=1; // 0 -> drag inner polygon's center
	    			// 1 -> drag outer polygon's center
	    			// 2 -> drag inner polygon vertex
	    			// 3 -> drag outer polygon vertex
	    			// 4 -> pick inner polygon vertices
	    			// 5 -> pick outer polygon vertices
	    			// 6 -> drag transect line's center
	    			// 7 -> drag transect line endpoint
	    			// 8 -> pick transect line endpoints
      
	boolean ifBoxIsVisible = true;
	boolean ifInnerBoxValid = true;
	boolean ifTransectLineVisible=true;
	double enlargeRatio = 1.1;
	double shrinkRatio = .9393636363;
	HistogramBox myBox=null;
	int scale=255; // tells how much total probe per pixel in histogram
		// region registers full scale in the plots.
	double pixelsPerMicrometer = 20.; // used to draw tic marks on profiles
	final static int MOUSE_REPORTS_LOCATION_COLORS = 12;
	final static int MOUSE_MOVES_OUTERBOX_CENTER = 23;
	final static int MOUSE_MOVES_INNERBOX_CENTER = 26;
	final static int MOUSE_CLICKS_DEFINE_POLYGON = 24;	
	final static int MOUSE_DRAGS_POLYGON_VERTEX = 25;	
	final static int MOUSE_CLICKS_DEFINE_TRANSECT = 28;	
	final static int MOUSE_DRAGS_TRANSECT_VERTEX = 29;	
	final static int MOUSE_MOVES_TRANSECT_CENTER = 27;
	JComboBox whichColors,whatMouseDoes,typeOfHistogram,innerouterBoxShapes,
		outerouterBoxShapes;
		
	public void resetSomeJLabels() {
		int selection;
		innerouterBoxShapes.setSelectedIndex(myBox.innerBoundary.boxShape-1);
		outerouterBoxShapes.setSelectedIndex(myBox.outerBoundary.boxShape-1);
		switch(histogramType) {
			case 8 :
			case 9 :
			case 10 :
				selection=histogramType-1;
				break;
			case 7 : // type 7 is last...
				selection=10;
				break;
			default :
				selection=histogramType;
				break;
			}
		typeOfHistogram.setSelectedIndex(selection-1);
		setCoreWanderingJButtonText(false);
/* histogramType 
 	1   "(conventional) in one section");
	2   "stacked totals in 3-D vol");
	3   "(conventional) in 3-D vol");	
	4   "profile in one section along transect");	
	5   "stacked 3-D profiles along transect");	
	6   "Zcore -discard pix dimmer than RGBdim");	
	8   "Zcore -discard RGBdim % of dimest pixels");
	9   "BleachCorrection -discard pix dimmer than RGBdim";
	10   "BleachCorrection -discard RGBdim % of dimest pixels";
	7   "full stats for callibration data pics");
		*/

			}

    public Stats(ShowPicJPanel picJPanel) {   
    super("statistics");
    this.picJPanel=picJPanel;
	WidgetMaker wmk = new WidgetMaker(this);
	Color bblue = new Color(90,90,255);
    JPanel panA,panC,panD,panE,panF,panG,midPan,borderPan,panH,panI,pan;
    getContentPane().setLayout(new BorderLayout()); 
    setSize(nomwidth, nomheight);
	setBackground(new Color(230,240,250));
	
	whichColors =  wmk.makeAJComboBox(new String [] {"Red only",
		"Green only","Blue only","all 3 RGB"});
	whatMouseDoes = wmk.makeAJComboBox(new String [] {"drag outer polygon's center",
		"drag outer polygon vertex",
		"pick outer polygon vertices",
		"drag inner polygon's center",
		"drag inner polygon vertex",
		"pick inner polygon vertices",
		"drag transect line's center",
		"drag transect line endpoint",
		"pick transect endpoints"});

	 typeOfHistogram = wmk.makeAJComboBox(new String [] {
 		"(conventional) in one section",
 		"stacked totals in 3-D vol",
 		"(conventional) in 3-D vol",
 		"profile in one section along transect",
 		"stacked 3-D profiles along transect",
 		"Zcore -discard pix dimmer than RGBdim",
 		"Zcore -discard RGBdim % of dimest pixels",
 		"BleachCorrection -discard pix dimmer than RGBdim",
 		"BleachCorrection -discard RGBdim % of dimest pixels",
 		"full stats for callibration data pics"});	
 		
	innerouterBoxShapes = wmk.makeAJComboBox(
		new String [] {"inner box","inner oval","inner polygon"});

	outerouterBoxShapes =  wmk.makeAJComboBox(
		new String [] {"outer box","outer oval","outer polygon"});
		
	Color textColor = Color.lightGray;
	threshRed=wmk.makeAJTextField("2",3,wmk.lightred);
	threshGreen=wmk.makeAJTextField("2",3,wmk.lightgreen);
	threshBlue=wmk.makeAJTextField("2",3,wmk.lightblue);
	zboxbot=wmk.makeAJTextField("  0",3,textColor);
	zboxtop=wmk.makeAJTextField(" 10",3,textColor);
	scaleTotalPerBoxPix = wmk.makeAJTextField(""+scale,5,textColor);
	areaPixels =  wmk.makeAJTextField("0000000",7,textColor);
	transectPixels = wmk.makeAJTextField("0000",4,textColor);
	pixPerMicron = wmk.makeAJTextField("22",4,textColor);
	transectWidth =  wmk.makeAJTextField("30",3,textColor);
	transectBlur = wmk.makeAJTextField("4",3,textColor);

// color secheme: A->0, B->c8, C->c2, D->c3, E->c4, F->c6, G->c7, H->c1, I->c5
	JPanel panA1 = new JPanel(); 
	pan=panA1;
	pan.setLayout(new GridLayout(0,3));
	pan.add(wmk.makeAJButton("read stats file"));
	pan.add(whichColors);
	pan.add(wmk.makeAJButton("make histogram"));
	
	panA = new JPanel(); 
	panA.setLayout(new GridLayout(0,1));
	panA.add(panA1);
	panA.add(typeOfHistogram);

	panB = new JPanel(); 
	pan=panB;
	pan.setLayout(new GridLayout(2, 2));
	pan.add(wmk.makeAJButton("activate mouse to "));
	pan.add(whatMouseDoes);
	globalLocalTog = wmk.makeAJButton("Core is right cylinder (toggle)");
	//  or                       "Core boxes wander (toggle)"
	pan.add(globalLocalTog);
	pinFreeCoreBoxToggle = wmk.makeAJButton("---------------------------");
	//          or                    "now interpolated; push to pin"
	//          or                    "now pinned; push to free it"
	pan.add(pinFreeCoreBoxToggle);
		
	panD = new JPanel();
	pan=panD;
	pan.setLayout(new GridLayout(1, 0));
	pan.add(innerouterBoxShapes);
	pan.add(wmk.makeAJButton("shrink "));
	pan.add(wmk.makeAJButton("enlarge "));

	panE = new JPanel();
	pan=panE;
	pan.setLayout(new GridLayout(1, 0));
	pan.add(outerouterBoxShapes);
	pan.add(wmk.makeAJButton(" shrink")); // note space preceeds 'shrink' for outer box
	pan.add(wmk.makeAJButton(" enlarge")); // note space preceeds 'enlarge' for outer box
			
	panF = new JPanel(); 
	pan=panF;
	pan.setLayout(new GridLayout(1, 0));
	pan.add(wmk.makeAJLabel("Rdim",JLabel.CENTER));
	pan.add(threshRed);
	pan.add(wmk.makeAJLabel("Gdim",JLabel.CENTER));
	pan.add(threshGreen);
	pan.add(wmk.makeAJLabel("Bdim",JLabel.CENTER));
	pan.add(threshBlue);

	
	panI = new JPanel();
	pan=panI;
	pan.setLayout(new GridLayout(1, 0));
	pan.add(wmk.makeAJLabel("TransLength",JLabel.RIGHT));
	pan.add(transectPixels);
	pan.add(wmk.makeAJLabel(" width=",JLabel.RIGHT));
	pan.add(transectWidth);
	pan.add(wmk.makeAJLabel(" blur=",JLabel.RIGHT));
	pan.add(transectBlur);

	panH = new JPanel();
	pan=panH;
	pan.setLayout(new GridLayout(1, 0));
// pix/micron  stackBot  stackTop  scale  boxArea  
	pan.add(wmk.makeAJLabel("pix/micron",JLabel.CENTER));
	pan.add(wmk.makeAJLabel("stackBot",JLabel.CENTER));
	pan.add(wmk.makeAJLabel("stackTop",JLabel.CENTER));
	pan.add(wmk.makeAJLabel("scale",JLabel.CENTER));
	pan.add(wmk.makeAJButton("boxArea"));

	panC = new JPanel(); 
	pan=panC;
	pan.setLayout(new GridLayout(1, 0));
	pan.add(pixPerMicron); 
	pan.add(zboxbot);
	pan.add(zboxtop);
	pan.add(scaleTotalPerBoxPix);
	pan.add(areaPixels);
	
	panG = new JPanel(); 
	pan=panG;
	pan.setLayout(new GridLayout(1, 0));
	pan.add(wmk.makeAJButton("outerBox on/off"));
	pan.add(wmk.makeAJButton("innerBox on/off"));
	pan.add(wmk.makeAJButton("transect on/off"));
			
	midPan = new JPanel();
	midPan.setLayout(new GridLayout(0, 1));
	midPan.add(panH);  // c1 labels: pix/micron  stackBot  stackTop  scale  boxArea
	midPan.add(panC);  // c2 textFields
	midPan.add(panD);  // c3 inner box ...
	midPan.add(panE);  // c4 outer box...
	midPan.add(panI);  // c5 transect params
	midPan.add(panF);  // c6 red thresh, green thresh,..
	midPan.add(panG);  // c7 innerBox on/ff, colors, visibility toggle
	
	getContentPane().add("North",panA); // c0 make histograms, type of histogram

	getContentPane().add("Center",midPan);
	getContentPane().add("South",panB); // c8 actinvate mouse, what mouse does
	
	myBox = new HistogramBox(picJPanel,this);
	myBox.setRasterLimits( 0, picJPanel.fullWidth, 0, picJPanel.fullHeight);
	picJPanel.primeMouseForTask(MOUSE_REPORTS_LOCATION_COLORS,this); // identify self
		//to picJPanel through which myBox draws();
	addWindowListener(this);
	readJTextFields();
  	}
  

	public void draw(Graphics g, int xorg, int yorg) {
		resetSomeJLabels();
		if(myBox != null)  {
			readJTextFields();
			myBox.draw(g,  xorg,  yorg);
			}
		}
		
//catch JComboBox events...	
	public void itemStateChanged(java.awt.event.ItemEvent event)
		{
		String arg = (String)(((JComboBox)(event.getSource())).getSelectedItem());
		myBox.makingHistograms=false;
        unPrimeMouse(); 
       	setCoreWanderingJButtonText(true);  // resets cylinderWarningCount to 0    
	    // choice items... 
      	if(arg.equals("drag transect line's center")) {
			mouseTask = 6;
			activateMouseDragging();
	    	}
       	if(arg.equals("drag transect line endpoint")) {
			mouseTask = 7;
			activateMouseDragging();
	    	}
       	if(arg.equals("pick transect endpoints")) {
			mouseTask = 8;
			activateMouseDragging();
	    	}
       	if(arg.equals("drag outer polygon's center")) {
			mouseTask = 1;
			activateMouseDragging();
	    	}
       	if(arg.equals("drag inner polygon's center")) {
			mouseTask = 0;
			activateMouseDragging();
	    	}
       	if(arg.equals("drag inner polygon vertex")) {
			mouseTask = 2;
			activateMouseDragging();
	    	}
      	if(arg.equals("drag outer polygon vertex")) {
			mouseTask = 3;
			activateMouseDragging();
	    	}
       	if(arg.equals("pick inner polygon vertices")) {
			mouseTask = 4;
			activateMouseDragging();
	    	}
      	if(arg.equals("pick outer polygon vertices")) {
			mouseTask = 5;
			activateMouseDragging();
	    	}
      	if(arg.equals("(conventional) in one section")) {
      		histogramType=1;
	    	}
      	if(arg.equals("stacked totals in 3-D vol")) {
      		histogramType=2;
	    	}
      	if(arg.equals("(conventional) in 3-D vol")) {
      		histogramType=3;
	    	}
	    	
      	if(arg.equals("profile in one section along transect")) {
      		histogramType=4;
	    	}
	    	
      	if(arg.equals("stacked 3-D profiles along transect")) {
      		histogramType=5;
	    	}
      	if(arg.equals("Zcore -discard pix dimmer than RGBdim")) {
      		histogramType=6;
	    	}
      	if(arg.equals("Zcore -discard RGBdim % of dimest pixels")) {
      		histogramType=8;
	    	}
      	if(arg.equals("BleachCorrection -discard pix dimmer than RGBdim")) {
      		histogramType=9;
	    	}
      	if(arg.equals("BleachCorrection -discard RGBdim % of dimest pixels")) {
      		histogramType=10;
	    	}

      	if(arg.equals("full stats for callibration data pics")) {
      		histogramType=7;
	    	}
	    	
      	if(arg.equals("Red only")) {
      		colorChoice=1;
      		talk("colorChoice=1");
	    	}
      	if(arg.equals("Green only")) {
      		colorChoice=2;
      		talk("colorChoice=2");
	    	}
      	if(arg.equals("Blue only")) {
      		colorChoice=3;
      		talk("colorChoice=3");
	    	}
      	if(arg.equals("all 3 RGB")) {
      		colorChoice=4;
      		talk("colorChoice=4");
	    	}	    	
      	if(arg.equals("outer oval")) {
			myBox.reshapeBox(true,2);
			readJTextFields();
			myBox.bndChg("outer oval"); 
			forcedRedraw();
	    	}
	    	
      	if(arg.equals("outer polygon")) {
			myBox.reshapeBox(true,3);
			readJTextFields();
			myBox.bndChg("outer polygon"); 
			forcedRedraw();
	    	}
	     
      	if(arg.equals("outer box")) {
			myBox.reshapeBox(true,1);
			readJTextFields();
			myBox.bndChg("outer box"); 
			forcedRedraw();
	    	}	
	    	 
      	if(arg.equals("inner oval")) {
			myBox.reshapeBox(false,2);
			readJTextFields();
			myBox.bndChg("inner oval"); 
			forcedRedraw();
	    	}
      	if(arg.equals("inner polygon")) {
			myBox.reshapeBox(false,3);
			readJTextFields();
			myBox.bndChg("inner polygon"); 
			forcedRedraw();
	    	}
      	if(arg.equals("inner box")) {
			myBox.reshapeBox(false,1);
			readJTextFields();
			myBox.bndChg("innerBox"); 
			forcedRedraw();
	    	}	 
      setCoreWanderingJButtonText(true); // resets cylinderWarningCount to 0   	
		}

    private void forcedRedraw() {
 		picJPanel.forceNewDrawing();
 		picJPanel.reDraw();
   		}

// catch button and menu events...
	public void actionPerformed(java.awt.event.ActionEvent event) {
		String arg = event.getActionCommand();
		// talk("Stats.actionPerformed GOT <"+arg+">");
		myBox.makingHistograms=false;
        unPrimeMouse(); 
 		picJPanel.showYourself();       
       	// menu items...
       	webCacheLevel(-1); // suspend web caching	
	    if(arg.equals("read stats file")) {
	    		ifReallyMakeHistogram=false;
	    		ArchiveMarks disker = new ArchiveMarks();   	
				Histogram anotherHistogram = new Histogram(this);
					// this will read in an old histogram from a disk file
				if(anotherHistogram != null && anotherHistogram.ifDiskReadWorked) {
					String redfn = anotherHistogram.redFilename;
					String greenfn = anotherHistogram.greenFilename;
					String bluefn = anotherHistogram.blueFilename;
					String hstName = anotherHistogram.myDiskName;
	    			adoptNewHistogram(anotherHistogram,false);
					makeAHistogram(redfn,greenfn,bluefn,hstName);
					}
				anotherHistogram=null;	
				disker = null;  
				resetSomeJLabels();
	    		}
	    		
			if(arg.startsWith("Core is right cylinder"))  { 
				initializeCoreStack(true);
				}											
			if(arg.startsWith("Core boxes wander"))  { 
				initializeCoreStack(false);
				}
			
       		setCoreWanderingJButtonText(true); // resets cylinderWarningCount to 0   	
			if(arg.equals("toggle visibility"))  {
					ifBoxIsVisible = !ifBoxIsVisible;
					tellBoxWhatsVisible();
					if(ifBoxIsVisible) talk("box is visible"); 
					else talk("don't draw boxes"); 
					readJTextFields();
					forcedRedraw();
					}
										
			if(arg.equals("make histogram"))  { 
	    		ifReallyMakeHistogram=true;
				makeAHistogram(null,null,null,null);
				}
					
			if(arg.equals("activate mouse to "))  {
				activateMouseDragging();
				}
			
			if(arg.equals("now interpolated; push to pin"))  { 
				manageCoreStack(true); // pin it down where it is
				}
			if(arg.equals("now pinned; push to free it"))  { 
				manageCoreStack(false); // free it; determine corners by interpolation
				}
			if(arg.equals("boxArea"))  { 
				myBox.makingHistograms=true;
				myBox.bndChg("boxArea button");
				computeArea();
				}

			if(arg.equals("innerBox on/off"))  { 
					ifInnerBoxValid = !ifInnerBoxValid;
					tellBoxWhatsVisible();
					myBox.bndChg("innerBox on/off"); 
					forcedRedraw();
					}
			if(arg.equals("outerBox on/off"))  { 
					ifBoxIsVisible = !ifBoxIsVisible;
					if(!ifBoxIsVisible)
						ifInnerBoxValid=false;
					tellBoxWhatsVisible();
					myBox.bndChg("outerBox on/off"); 
					forcedRedraw();
					}
			if(arg.equals("transect on/off"))  { 
					ifTransectLineVisible = !ifTransectLineVisible;
					tellBoxWhatsVisible();
					forcedRedraw();
					}
					
			if(arg.equals("enlarge "))  { 
					innerBoxOnOff(true);
					myBox.rescaleBoundary(false,enlargeRatio);
					innerBoxOnOff(true);
					forcedRedraw();
					}
						
			if(arg.equals("shrink "))  { 
					innerBoxOnOff(true);
					myBox.rescaleBoundary(false,shrinkRatio);
					forcedRedraw();
					}

			if(arg.equals(" enlarge"))  { 
					outerBoxOnOff(true);
					myBox.rescaleBoundary(true,enlargeRatio);
					forcedRedraw();
					}
			if(arg.equals(" shrink"))  { 
					outerBoxOnOff(true);
					myBox.rescaleBoundary(true,shrinkRatio);
					forcedRedraw();
					}
       	webCacheLevel(0); // resume web caching	
		}
	
// catch WindowListener events  
	public void windowActivated(java.awt.event.WindowEvent event) {
		// talk("window activated");
		}
	public void windowDeactivated(java.awt.event.WindowEvent event) {
		//  talk("window Deactivated");
		}
	public void windowOpened(java.awt.event.WindowEvent event) {}
	public void windowIconified(java.awt.event.WindowEvent event) {}
	public void windowDeiconified(java.awt.event.WindowEvent event) {}
	public void windowClosing(java.awt.event.WindowEvent event) {
	    setVisible(false);
		}
	public void windowClosed(java.awt.event.WindowEvent event) {
		}
	 
 	 public void setRGBdimText(int rd, int gd, int bd) {
 	 	Rthresh=rd;
		threshRed.setText(""+Rthresh);
		Gthresh=gd;
		threshGreen.setText(""+Gthresh);
		Bthresh=bd;
		threshBlue.setText(""+Bthresh);
 	 	}
 	 	
 	 public void setRGBdimText(int [] RGBdims) {
 		setRGBdimText(RGBdims[0], RGBdims[1], RGBdims[2]);
 		}
	 	   	
    public int [] getRGBdimValues() {
    	int [] rgbdims = new int[3];
 		rgbdims[0] = (int)(Double.valueOf(threshRed.getText()).doubleValue());	
		rgbdims[1] = (int)(Double.valueOf(threshGreen.getText()).doubleValue());	
		rgbdims[2] = (int)(Double.valueOf(threshBlue.getText()).doubleValue());
		myBox.takeNewRGBdims(rgbdims);
		return rgbdims;
   		}	
   		
    private void setRGBdimsFromTextfields() {
    	int [] rgbdims=getRGBdimValues(); 
		Rthresh = rgbdims[0];	
		Gthresh = rgbdims[1];	
		Bthresh = rgbdims[2];
    	}
    	
     public void adoptNewHistogram(Histogram  ahist, boolean ifComputeArea) {
    	ahist.myBox.transferDataTo(myBox);
		zmin = ahist.zmin;		zboxbot.setText(""+zmin);
		zmax = ahist.zmax;	zboxtop.setText(""+zmax);

		setRGBdimText(ahist.Rthresh, ahist.Gthresh, ahist.Bthresh);
		colorChoice=ahist.colorChoice;
		whichColors.setSelectedIndex(colorChoice-1);
		histogramType=ahist.histogramType;  
		typeOfHistogram.setSelectedIndex(histogramType-1);
		picJPanel.gotosection(ahist.ST_curJFrame); 		
		ifBoxIsVisible = ahist.ST_ifBoxIsVisible;
		ifInnerBoxValid = ahist.ST_ifInnerBoxValid;
		innerBoxOnOff(ifInnerBoxValid);
		ifTransectLineVisible=ahist.ST_ifTransectLineVisible;
		
		enlargeRatio =  ahist.ST_enlargeRatio;
		shrinkRatio =  ahist.ST_shrinkRatio;
		tranwidth=ahist.ST_width; 	transectWidth.setText(""+tranwidth);
		tranblur=ahist.ST_blur;		transectBlur.setText(""+tranblur);
					
		scale= ahist.scale; 
		scaleTotalPerBoxPix.setText(""+scale);
		
		pixelsPerMicrometer =  ahist.pixelsPerMicrometer;
		pixPerMicron.setText(""+pixelsPerMicrometer);
		
		transectWidth.setText(""+(myBox.myTransect.width));
		transectBlur.setText(""+(myBox.myTransect.blur));
		tellBoxWhatsVisible();
		if(ifComputeArea) computeArea();
		myBox.myTransect.rescale(1.0);
    	}
    
    private void readJTextFields() {
   		int dumx,dumy;
   		String stg;
   		if(picJPanel != null && picJPanel.moviePlaying) return;
		setRGBdimsFromTextfields();
		zmin = 	(int)(Double.valueOf(zboxbot.getText()).doubleValue());
		zmax = 	(int)(Double.valueOf(zboxtop.getText()).doubleValue());
		if(zmin>zmax) {
			dumx=zmax;
			zmax=zmin;
			zmin=dumx;
			zboxbot.setText(""+zmin);
			zboxtop.setText(""+zmax);
			}
		scale = (int)(Double.valueOf(scaleTotalPerBoxPix.getText()).doubleValue());
		pixelsPerMicrometer = Double.valueOf(pixPerMicron.getText()).doubleValue(); 
		if(pixelsPerMicrometer<4) {
			talk("Warning.. pixelsPerMicrometer = "+pixelsPerMicrometer+" repaired to 22.5");
			pixelsPerMicrometer=22.5;
			}
		pixPerMicron.setText(""+pixelsPerMicrometer);
		tranwidth = (int)(Double.valueOf(transectWidth.getText()).doubleValue());
		tranblur = (int)(Double.valueOf(transectBlur.getText()).doubleValue());
		myBox.setParams(tranwidth,tranblur,pixelsPerMicrometer);
		}
	
	public void webCacheLevel(int lev) {
		picJPanel.webCacheLevel(lev);
		}
		
	private boolean makeAHistogram(String redfn, String greenfn, String bluefn,String hstName) {
    	Histogram anotherHistogram;
    	int singleJFrame;
		readJTextFields(); 
		myBox.makingHistograms=true;
		// caching web stacks will slow things down...so supress it, then restore it when done.
		
		switch (histogramType) {
			case 1 :
				singleJFrame=picJPanel.curStack.curJFrame;
				anotherHistogram = new Histogram(histogramType,singleJFrame,singleJFrame,colorChoice,
							Rthresh, Gthresh, Bthresh, scale,pixelsPerMicrometer,this,redfn,greenfn,bluefn,hstName);
				break;						
				case 2:  // stacked histograms for 3-D volume
				case 3: // conventional histogram for 3-D volume
					anotherHistogram=new Histogram(histogramType,zmin,zmax,colorChoice,
						Rthresh, Gthresh, Bthresh, scale,pixelsPerMicrometer,this,redfn,greenfn,bluefn,hstName);
					break;	
				case 4 :
					singleJFrame=picJPanel.curStack.curJFrame;
					anotherHistogram=new Histogram(histogramType,singleJFrame,singleJFrame,colorChoice,
						Rthresh, Gthresh, Bthresh, scale,pixelsPerMicrometer,this,redfn,greenfn,bluefn,hstName);
					break;						
				case 5:  // stacked profiles along transect
				case 6:  // z-core profile thru a 3-D stackwith Rdim,Gdim, and Bdim dim abs thresholds.
				case 8:  // z-core profile thru a 3-D stack with Rdim,Gdim, and Bdim as percentageDarkDiscarded.
				case 9:  // bleach correction drop all pixels dimmer than RGBdims 
				case 10: // bleach correction RGBdims are percentage pf darkest pixels to discard
					talk("Rthresh="+Rthresh);
					anotherHistogram=new Histogram(histogramType,zmin,zmax,colorChoice,
						Rthresh, Gthresh, Bthresh, scale,pixelsPerMicrometer,this,redfn,greenfn,bluefn,hstName);
					break;	
				case 7: //do full stats on a directory full of callibration standards pics
					talk("histogramType==7 in Stats.makeAHistogram(). Inconceivable");
					anotherHistogram=new Histogram(this, pixelsPerMicrometer, hstName);
					break;
				default:
					return false;
				}
			anotherHistogram.copyDataFrom(this);
			return true;
		}
		
public void sectionChangedAlert() { // called principally from the BioradCanvas object
	int atframe = picJPanel.curStack.curJFrame;
	int nsects = picJPanel.curStack.NJFrames;
 	if(!myBox.ifCoreBoxesWander) {// do nothing
		myBox.setSection(atframe); // do nothing else
 		return;
 		}
	if(atframe>=0 && atframe < nsects)
 		manageCoreStack(myBox.ifCoreBoxPinned[atframe]);
 	else 
		myBox.setSection(atframe); 
 	}
	
public void manageCoreStack(boolean pinFreeSet) {
	int atframe = picJPanel.curStack.curJFrame;
	int nsects = picJPanel.curStack.NJFrames;
	if(!myBox.ifCoreBoxesWander || myBox.ifCoreBoxPinned==null) {
		myBox.setSection(atframe); // do nothing else
		forcedRedraw();
 		return;
 		}

	myBox.ifCoreBoxPinned[atframe]=pinFreeSet;
	myBox.setSection(atframe);				
	if(!myBox.ifCoreBoxPinned[atframe]) 
		pinFreeCoreBoxToggle.setText("now interpolated; push to pin");				
	else 
		pinFreeCoreBoxToggle.setText("now pinned; push to free it");
	resetSomeJLabels();
	forcedRedraw();
}

public void initializeCoreStack(boolean ifwander) {
	int atframe = picJPanel.curStack.curJFrame;
	int nsects = picJPanel.curStack.NJFrames;
	if(++cylinderWarningCount>=4) {
		cylinderWarningCount=0;
		if(ifwander) { // initialize to allow core to wander
			myBox.makeNewCoreBoundaryStacks(nsects);
			pinFreeCoreBoxToggle.setText("now interpolated; push to pin");
			sectionChangedAlert();
			}			
		else {// initialize to preclude core wandering
			myBox.destroyCoreBoundaryStacks();
			pinFreeCoreBoxToggle.setText("------------------------------");
			}
		}
	setCoreWanderingJButtonText(false);
}

private void setCoreWanderingJButtonText(boolean ifResetWarnings) {
	String [] togs = { "(toggle)","(change?)","CHANGE?","UN DO THIS"};
	Color [] warningColor = {new Color(115,185,105),Color.orange,Color.red,Color.yellow};
	if(ifResetWarnings) {
		talk("reset warning count to 0 ");
		cylinderWarningCount=0;
		}
	panB.setBackground(warningColor[cylinderWarningCount]);			
	if(myBox.ifCoreBoxesWander) 
		globalLocalTog.setText("Core boxes wander "+togs[cylinderWarningCount]);
	else	
		globalLocalTog.setText("Core is right cylinder "+togs[cylinderWarningCount]);
}
		
	private boolean activateMouseDragging() {
			readJTextFields();
				switch (mouseTask) {
					case 4:
						talk("click to define polygon vertices. Clicking near first vertex");
						talk("closes the polygon... "); 
						innerBoxOnOff(true); 
						myBox.newInnerPolygon();
						picJPanel.primeMouseForTask(MOUSE_CLICKS_DEFINE_POLYGON,this);
						break;
					case 5:
						talk("click to define polygon vertices. Clicking near first vertex");
						talk("closes the polygon... "); 
						myBox.newOuterPolygon();
						outerBoxOnOff(true);
						picJPanel.primeMouseForTask(MOUSE_CLICKS_DEFINE_POLYGON,this);
						break;
					case 8:
						talk("click to define transect line endpoints."); 
						myBox.newTransectLine();
						transectLineOnOff(true);
						picJPanel.primeMouseForTask(MOUSE_CLICKS_DEFINE_TRANSECT,this);
						break;
					case 1 : // mouse drags outer box center... 
						talk("next click in micrograph window moves center of outer histogram boundary"); 
						picJPanel.primeMouseForTask(MOUSE_MOVES_OUTERBOX_CENTER,this);
						outerBoxOnOff(true);
						break;
					case 0 : // mouse drags inner box center 
						talk("next click in micrograph window moves center of inner histogram boundary"); 
						picJPanel.primeMouseForTask(MOUSE_MOVES_INNERBOX_CENTER,this);
						innerBoxOnOff(true); 
						picJPanel.parent.setVisible(true);
						picJPanel.requestFocus();
						break;
					case 6 : // mouse drags transect line center
						talk("next click in micrograph window moves center of transect line"); 
						picJPanel.primeMouseForTask(MOUSE_MOVES_TRANSECT_CENTER,this);
						transectLineOnOff(true);
						break;
					case 2: // drag inner polygon vertex
						talk("click near vertex you want to move and drag it");
						innerBoxOnOff(true);
						primeMouseToDragPolygonVertex();
						break;
					case 3: // drag outer polygon vertex
						talk("click near vertex you want to move and drag it");
						outerBoxOnOff(true);
						primeMouseToDragPolygonVertex();
						break;
					case 7: // drag transect line vertex
						talk("click near transect line end you want to move and drag it");
						transectLineOnOff(true);
						myBox.primeToDragVertex(mouseTask);
						picJPanel.primeMouseForTask(MOUSE_DRAGS_TRANSECT_VERTEX,this);
						break;
					default: return false;	
					}
				picJPanel.parent.setVisible(true);
				picJPanel.requestFocus();
				return true;
				}
				
	public void primeMouseToDragPolygonVertex() {
		myBox.primeToDragVertex(mouseTask);
		picJPanel.primeMouseForTask(MOUSE_DRAGS_POLYGON_VERTEX,this);
		picJPanel.parent.setVisible(true);
		picJPanel.requestFocus();
		}
		
	private void unPrimeMouse() {
		picJPanel.primeMouseForTask(MOUSE_REPORTS_LOCATION_COLORS,this);
		}
		
	private void tellBoxWhatsVisible() {
		myBox.ifBoxIsVisible = ifBoxIsVisible;
		myBox.ifInnerBoxValid = ifInnerBoxValid;
		myBox.ifTransectLineVisible = ifTransectLineVisible;
		}
		
	private void innerBoxOnOff(boolean ifon) {
		ifInnerBoxValid = ifon;
		if(ifon) 
			ifBoxIsVisible = true;
		tellBoxWhatsVisible();
		}
		
	private void outerBoxOnOff(boolean ifon) {
		ifBoxIsVisible = ifon;
		if(!ifon) 
			ifInnerBoxValid = false;
		tellBoxWhatsVisible();
		}
		
	private void transectLineOnOff(boolean ifon) {
		ifTransectLineVisible=ifon;
		tellBoxWhatsVisible();
		}
		
	private int getThresh(JTextField threshold) {
	    int thrs = (int)(Double.valueOf(threshold.getText()).doubleValue());
	    return thrs;
		}
		
	public void changeJPanelSize(double ratio) {
		int pwidth = getSize().width;
		int pheight = getSize().height;
		pwidth = (int)(ratio*pwidth +.4999);
		pheight = (int)(ratio*pheight +.4999);
		setSize(pwidth,pheight);
		show();
		}
		
	public void moveVertex(double [] mouseLoc, int doWhat) {
			myBox.moveVertex((int)(mouseLoc[0]+.499), (int)(mouseLoc[1]+.499),doWhat);
	    	forcedRedraw();
	    	}
			
	public boolean nextVertexOfPolygon(double [] mouseLoc, int doWhat) {
		boolean retval;
		retval=myBox.nextVertexOfPolygon(mouseLoc,doWhat);
		forcedRedraw();
		return(retval);
		}

	public void moveBoxCenter(double [] mouseLoc, int doWhat) {
	    	int xcen = (int)(mouseLoc[0]+.499);
	    	int ycen = (int)(mouseLoc[1]+.499);
	    	readJTextFields(); 
	    	if(doWhat==MOUSE_MOVES_OUTERBOX_CENTER)
	    		myBox.outerBoundary.moveCenterTo(xcen,ycen);
	    	if(doWhat==MOUSE_MOVES_INNERBOX_CENTER)
	    		myBox.innerBoundary.moveCenterTo(xcen,ycen);	    	
	    	if(doWhat==MOUSE_MOVES_TRANSECT_CENTER)
	    		myBox.myTransect.moveCenterTo(xcen,ycen);	    	
	    	forcedRedraw();
	    	}
			
	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}

	public void computeArea() {
		readJTextFields();
		myBox.myTransect.rescale(1.0);
		myBox.resetVoxPointers(); // to tell area
		forcedRedraw();
		}

}
	
