package staxx;

/* Info ...  author, copyright, bundle-type, etc */
/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
import java.awt.*;  import javax.swing.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;


class StereoPairCanvas extends Canvas{	
	
	StereoPairFrame parent=null;
	int [] rayTraceRaster=null, maxProjectionRaster=null, blendedRaster=null;	
	double blendFrac=.35,oldBlendFrac=-1.;	
	int blendedRasterWidth=0,blendedRasterHeight=0,npix=0;
	int [] scaledPic=null;
	int scaledPicWidth=0, scaledPicHeight=0;
	double scaleFactor = .5; // scale factor for the picture actually displayed.
	double oldScaleFactor=-1.;
	Image theImage=null;
	boolean scaledPicReady=false;
	boolean ifSwapEyes = false, dontPaint=false, neverBlendAgain=false;;
	static final int redmask=(255 << 16), greenmask = (255<<8), bluemask = 255;		     	
	static final int alpha = (255<<24);
	static final double brightenFactor=1.1;
 
 	public void sepuku() {
		dontPaint=true;
		rayTraceRaster=null;
		maxProjectionRaster=null;
		blendedRaster=null;
		scaledPic=null;
		parent=null;
		setVisible(false);
		}
		
	private boolean blendProjections() {
		if(neverBlendAgain)
			return false;
		if(rayTraceRaster == null) {
			blendedRaster = maxProjectionRaster;
			neverBlendAgain=true; // there is nothing to blend
			return true;
			}
			
		if(maxProjectionRaster == null) {
			blendedRaster = rayTraceRaster;
			neverBlendAgain=true; // there is nothing to blend
			return true;
			}
		if(blendFrac != oldBlendFrac) {
			if(blendFrac<0)
				blendFrac=0;
			if(blendFrac > 1)
				blendFrac=1;
			oldBlendFrac = blendFrac;
			double dualBLendFrac = 1 - blendFrac;
			if(blendedRaster == null)
				blendedRaster = new int[npix];
			int i,red,green,blue,maxpxl,raypxl;
			for(i=0;i<npix;i++) {
				maxpxl = (maxProjectionRaster[i]&redmask)>>16;
				raypxl = (rayTraceRaster[i]&redmask)>>16;				
				red = (int)(blendFrac*maxpxl + dualBLendFrac*raypxl);
				
				maxpxl = (maxProjectionRaster[i]&greenmask)>>8;
				raypxl = (rayTraceRaster[i]&greenmask)>>8;				
				green = (int)(blendFrac*maxpxl + dualBLendFrac*raypxl);
				
				maxpxl = (maxProjectionRaster[i]&bluemask);
				raypxl = (rayTraceRaster[i]&bluemask);				
				blue = ((int)(blendFrac*maxpxl + dualBLendFrac*raypxl));
				blendedRaster[i] = alpha | (red<<16) | (green<<8) | blue;
				}
			return true;				
			}
		return false;			
		}
		
	public StereoPairCanvas(StereoPairFrame parent, int origWidth,int origHeight, int [] rayTraceRas,
				int [] maxProjectionRas,  double blendFrac, double scaleFac, int stereoSeparation) {
		this.parent=parent;
		this.blendFrac = blendFrac;
		ifSwapEyes = stereoSeparation < 0;
		blendedRasterWidth=origWidth;
		blendedRasterHeight = origHeight;
		int i;
		npix = blendedRasterWidth*blendedRasterHeight;
		if(rayTraceRas != null) {
			rayTraceRaster = new int[npix];
			for(i=0;i<npix;i++)
				rayTraceRaster[i] = rayTraceRas[i];
			}
		if(maxProjectionRas != null) {
			maxProjectionRaster = new int[npix];
			for(i=0;i<npix;i++)
				maxProjectionRaster[i] = maxProjectionRas[i];
			}
			
		blendProjections();
		scaleFactor = scaleFac;	
		rescaleImage();
		setVisible(true);
		}


	public void forceRescaleImage() {
		if(dontPaint) return;
		oldScaleFactor=-1.;
		rescaleImage();
		}

	public void flipEyes() { 
		if(dontPaint) return;
		int ipix,ihalf=scaledPicWidth/2,i,ir,il,row,rowoff=0,rightmarg = scaledPicWidth -1;
		for(row=0;row<scaledPicHeight;row++) {
			for(i=0;i<ihalf;i++) {
				il = rowoff + i;
				ir = il+ihalf;
				ipix = scaledPic[il];
				scaledPic[il] = scaledPic[ir];
				scaledPic[ir] = ipix;
				}
			rowoff += scaledPicWidth;
			}
		if(dontPaint) return;
		theImage = createImage(new MemoryImageSource(scaledPicWidth, scaledPicHeight,
			 	ColorModel.getRGBdefault(), scaledPic, 0, scaledPicWidth)); 
		if(dontPaint) return;
		scaledPicReady=true;
		parent.repaint();
		}
		
		
	public void rescaleImage() {
		if(!blendProjections() && oldScaleFactor==scaleFactor) {
			talk("no need. Current scale factor is already == "+scaleFactor);
			return;
			}
		oldScaleFactor = scaleFactor;
		int [][][] A = parent.parent.picJPanel.curStack.getTriColorLUTS();
	    int redchan,greenchan,bluechan,apix; 
		scaledPicWidth = (int)(.49 + scaleFactor*blendedRasterWidth);
		scaledPicHeight = (int)(.49 + scaleFactor*blendedRasterHeight);
		talk("Rescaling to factor "+scaleFactor+" which gives a width of "+scaledPicWidth+" and a height of "+scaledPicHeight);
		scaledPic = new int[scaledPicWidth*scaledPicHeight];
		int row,col,red,green,blue,px;
		/* the mapping from the new picture to the old one is
			xorig = xscale/scaleFactor
			yorig = yscale/scaleFactor */
		double xorig,yorig;
		double dx,dxm,dy,dym;
		int ixo,ixop,iyo,iyop,pixoo,pixop,pixpo,pixpp,iyoOffset,iyopOffset,pxoo,pxop,pxpo,pxpp,coladd,
			halfScaledWidth = scaledPicWidth/2;

		for(col=0; col<scaledPicWidth; col++) {
		 if(ifSwapEyes) 
		  		coladd = col >= halfScaledWidth? -halfScaledWidth : halfScaledWidth;
		 else
		 	coladd=0; 	
		 		
		for(row=0; row<scaledPicHeight; row++) {
				xorig = col/scaleFactor;
				yorig = row/scaleFactor;
				ixo = (int)xorig;
				if((ixop=ixo+1) > blendedRasterWidth-1) 
					ixop=ixo;
				iyo=(int)yorig;
				iyoOffset = iyo*blendedRasterWidth;
				if((iyop= iyo+1) > blendedRasterHeight-1)
					iyop = iyo;
				iyopOffset = iyop*blendedRasterWidth;
				dx = (xorig-ixo);
				dy = (yorig-iyo);
				dxm=1. - dx;
				dym=1. - dy;
				pixoo = blendedRaster[iyoOffset+ixo];
				pixop = blendedRaster[iyoOffset+ixop];
				pixpo = blendedRaster[iyopOffset+ixo];
				pixpp = blendedRaster[iyopOffset+ixop];
				
				pxoo = pixoo&redmask; pxop=pixop&redmask;
				pxpo = pixpo&redmask; pxpp = pixpp&redmask;				
				redchan = ((int)(dym*(dxm*pxoo + dx*pxop) + dy*(dxm*pxpo + dx*pxpp)))>>16;
				if((redchan = (int)(parent.redChannelBoost*redchan)) > 255)
					redchan=255;
				
				pxoo = pixoo&greenmask; pxop=pixop&greenmask;
				pxpo = pixpo&greenmask; pxpp = pixpp&greenmask;				
				greenchan = ((int)(dym*(dxm*pxoo + dx*pxop) + dy*(dxm*pxpo + dx*pxpp)))>>8;
				if((greenchan = (int)(parent.greenChannelBoost*greenchan)) > 255)
					greenchan=255;
				

				pxoo = pixoo&bluemask; pxop=pixop&bluemask;
				pxpo = pixpo&bluemask; pxpp = pixpp&bluemask;				
				bluechan = (int)(dym*(dxm*pxoo + dx*pxop) + dy*(dxm*pxpo + dx*pxpp)); 
				if((bluechan = (int)(parent.blueChannelBoost*bluechan)) > 255)
					bluechan=255;
				
					/* apix = blendedRaster[iyoOffset+ixo];
	     			redchan = (apix & redmask)>>16;
	     			greenchan = (apix & greenmask)>>8;
	     			bluechan = apix & bluemask; */
				pxpp = alpha | (max3(A[0][0][redchan], A[1][0][greenchan], A[2][0][bluechan]) << 16) | 
					(max3(A[0][1][redchan], A[1][1][greenchan], A[2][1][bluechan]) << 8) | 
						max3(A[0][2][redchan], A[1][2][greenchan], A[2][2][bluechan]);

				scaledPic[row*scaledPicWidth + col + coladd] = pxpp;				
				}}
				
		theImage = createImage(new MemoryImageSource(scaledPicWidth, scaledPicHeight,
			 	ColorModel.getRGBdefault(), scaledPic, 0, scaledPicWidth)); 
		scaledPicReady=true;
		parent.repaint();
		}

	private int max3(int one, int two, int three) {
		if(one<two) one=two;
		if(one<three) return (int)three;
		else return (int)one;
		}
		
		

	public String writeJpeg(int [] anIm, int itswidth, int itsheight, String filename) {   
		if(dontPaint) return null;
		  GMORasFuncs grf = new GMORasFuncs();
			try{
				if(filename==null)
					return (grf.writeJpeg(itsheight,itswidth,anIm,1));
				else {
					grf.writeJpeg(filename,itsheight,itswidth,anIm,1);
					return filename;
					}
				}
			  catch(IOException e) {
				talk("Oops, writeJpeg(...) failed ");
				}
			return null;
			}
	
	public String scaledPic2jpg(String filename) {
		if(dontPaint) return null;
		return writeJpeg(scaledPic, scaledPicWidth, scaledPicHeight, filename);
		}
						
	public String scaledLeftPic2jpg(String filename) {
		if(dontPaint) return null;
		int row,col,scaledHalfPicWidth = scaledPicWidth/2;
		int [] halfPic = new int[scaledHalfPicWidth*scaledPicHeight];
		for(row=0; row<scaledPicHeight;row++) 
			for(col=0;col<scaledHalfPicWidth;col++) 
				halfPic[row*scaledHalfPicWidth + col] = scaledPic[row*scaledPicWidth + col];
		return writeJpeg(halfPic, scaledHalfPicWidth, scaledPicHeight, filename);
		}
						
	public void paint(Graphics g) {
		if(dontPaint) return;
		if(scaledPicReady) {
			try {
				g.drawImage(theImage, 0, 0, scaledPicWidth, scaledPicHeight, parent);
			 	//g.drawImage(theImage,ColorModel.getRGBdefault(), scaledPic, 0, scaledPicWidth)), 0, 0, scaledPicWidth, scaledPicHeight, parent);
			 	}
		  	catch (Exception e) {
				System.out.println("Error painting theImage panel: " + e.toString()+", parent = "+parent);
				}
			}
		if(dontPaint) return;
		setSize(scaledPicWidth, scaledPicHeight);
		setVisible(true);
		parent.parent.scaleFactor = scaleFactor;
		}

	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}

}