package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.*;
import java.util.*;
import java.awt.*;  import javax.swing.*;

class SwarmOfPoints { // a swarm of contiguous SwarmPoints 
	int ncols;  // number of columns in its raster
	int nrows;  //  number of columns in its raster
	SwarmPoint firstPoint;
	SwarmPoint lastPoint;
	short [] ras1; // the first raster, usually the one we started with
	short [] ras2; // the first raster, usually the one we started with
	SwarmOfPoints nextSwarm=null;
	int [][] locations;
	/* locations[0][0] is min x-coordinate, locations[0][1] is max x-coordinate,
					locations[0][2] is x-coordinate of center of gravity,
					and locations[0][3] is the x-span of the swarm.
	  locations[1][0] is min y-coordinate, locations[1][1] is max y-coordinate,
					locations[1][2] is y-coordinate of center of gravity,
					and locations[1][3] is the y-span of the swarm.
	  locations[2][0] is min z-coordinate, locations[2][1] is max z-coordinate,
					locations[2][2] is z-coordinate of center of gravity,
					and locations[2][3] is the z-span of the swarm.
	 */
	int diameter=0, numPoints=0;	
	// the locations of the swarm
	
	public SwarmOfPoints(short [] originalRaster, int ncols, int nrows, 
		int xison, int yison, short [] tempRaster) { // initialize swarm with known point that's on at
			// (xison,yison) 
		this.ncols=ncols;
		this.nrows=nrows;
		ras1 = originalRaster;
		ras2 = tempRaster;
		firstPoint = new SwarmPoint(xison,yison,0,this);
		firstPoint.neighborPoint=null;
		lastPoint = firstPoint;
		firstPoint.migrate(ras1,ras2);
		numPoints=1;
		locations = new int[3][4];
		locations[0][2]=0;
		locations[1][2]=0;
		locations[2][2]=0;
		}

	public void collectConnectedSwarm() { 
		// find all points connected to current points in the swarm
		// and in the process, move each point found from ras1 to ras2.
		SwarmPoint apoint = firstPoint, bpoint;
		while(apoint != null) {
			while( (bpoint = apoint.locateNeighbor(ras1)) != null) {
				++numPoints;
				lastPoint.neighborPoint = bpoint; // add bpoint onto end of linked list
				bpoint.migrate(ras1,ras2);
				lastPoint=bpoint; // bpoint becomes new lastPoint.
				} 
			// when fall thru, all of apoint's neighbors have been found 
			// and have migrated from ras1 to ras2.
			apoint = apoint.neighborPoint;
			}
		}
					
	public int getLocations() { // returns the diameter
		locations[0][0] = firstPoint.x; locations[0][1]=firstPoint.x; 
		locations[1][0] = firstPoint.y; locations[1][1]=firstPoint.y;
		locations[2][0] = firstPoint.z; locations[2][1]=firstPoint.z;
		locations[0][2] = firstPoint.x;
		locations[1][2] = firstPoint.y;
		locations[2][2] = firstPoint.z;
		int npts=1,i;
		SwarmPoint apoint = firstPoint;
		while((apoint = apoint.neighborPoint) != null) {
			if(apoint.x<locations[0][0]) locations[0][0]=apoint.x;
			if(apoint.x>locations[0][1]) locations[0][1]=apoint.x;
			if(apoint.y<locations[1][0]) locations[1][0]=apoint.y;
			if(apoint.y>locations[1][1]) locations[1][1]=apoint.y;
			if(apoint.z<locations[2][0]) locations[2][0]=apoint.z;
			if(apoint.z>locations[2][1]) locations[2][1]=apoint.z;
			++npts;
			locations[0][2] += apoint.x;
			locations[1][2] += apoint.y;
			locations[2][2] += apoint.z;
			}
		numPoints=npts;
		diameter=0;
		for(i=0;i<3;i++) {
    		locations[i][2] = (int)(.499 + (double )locations[i][2]/(double )npts); /*x CofG */
			if((locations[i][3]=locations[i][1]-locations[i][0])>diameter)
				diameter=locations[i][3];
			}
		return diameter;
		}

	public SwarmOfPoints migrateAndDissolve(short [] fromras, short [] toras) {
	// transfers the swarm from fromras  to toras, and then free its storage
		SwarmPoint apoint = firstPoint,bpoint;
		while(apoint != null) {	
			apoint.migrate(fromras,toras);
			bpoint=apoint;
			apoint = bpoint.neighborPoint;
			bpoint.sepuku();
			}
		ras2=null;
		firstPoint=null;
		lastPoint=null;
		ras1=null;
		ras2=null;
		return nextSwarm; // which, if null, means there are no more swarms.
		}	
		
	public SwarmOfPoints vacateAndDissolve(short [] fromras) {
	// removes the swarm from fromras, and then free its storage, returns next swarm of points
		SwarmPoint apoint = firstPoint,bpoint;
		while(apoint != null) {	
			apoint.vacate(fromras);
			bpoint=apoint;
			apoint = bpoint.neighborPoint;
			bpoint.sepuku();
			}
		ras2=null;
		ras1=null;
		firstPoint=null;
		lastPoint=null;
		return nextSwarm; // which, if null, means there are no more swarms.
		}	
		
	public SwarmOfPoints dissolve() {
		SwarmPoint apoint = firstPoint,bpoint;
		while(apoint != null) {	
			bpoint=apoint;
			apoint = bpoint.neighborPoint;
			bpoint.sepuku();
			}
		ras1=null;
		ras2=null;
		firstPoint=null;
		lastPoint=null;
		return nextSwarm; // which, if null, means there are no more swarms.
		}	
		
	public void sepuku() {
		firstPoint=null;
		lastPoint=null;
		ras1=null;
		ras2=null;
		nextSwarm=null;
		}
		
}	
		
