package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.*;
import java.util.*;
import java.awt.*;  import javax.swing.*;

class SwarmPoint { // the typical point belonging to a connected piece
	int x;  // its x coordinate
	int y;  // its y coordinate
	int z;  // its z coordinate
	SwarmPoint neighborPoint;
	SwarmOfPoints parent=null;
	int checked = 0; /* 0 means no neighboring positions checked
		                1 means neighbor at x+1,y checked
		                2 means neighbor at x+1,y+1 checked
		                3 means neighbor at x,y+1 checked
		                4 means neighbor at x-1,y+1 checked
		                5 means neighbor at x-1,y checked
		                6 means neighbor at x-1,y-1 checked
		                7 means neighbor at x,y-1 checked
		                8 means neighbor at x+1,y-1 checked
		                -1 means all neighbors checked. 
like this...		                
		                
		         6       7        8
		         5       0        1
		         4       3        2  */
		         
		         
	public void sepuku() {
		neighborPoint=null;
		parent=null;
		}
			         
	public SwarmPoint(int x, int y, int z, SwarmOfPoints swarm) {
		this.x=x;
		this.y=y;
		this.z=z;
		parent=swarm;
		neighborPoint=null;
		checked=0;
		}
	
	public SwarmPoint locateNeighbor(short [] ras) {
		/* checks out neighboring locations sequentially in clockwise order, returning
		 null when no more neighbors exist, and returning a SwarmPoint when an
		 occupied neighbor exists containing that neighbor. */
		 int xx=x,yy=y;
		 if(checked == -1) return null;
		 while(++checked < 9) {
		   switch (checked) {
		 	case 1:
		 		xx=1+x; yy=y;      break;
		 	case 2 :
		 		xx=1+x; yy=1+y; break;
		 	case 3 :
		 		 xx=x; yy=1+y; break;
		 	case 4 :
		 		xx=x-1; yy=1+y; break;
		 	case 5 :
		 		xx=x-1; yy=y;     break;
		 	case 6 :
		 		xx=x-1; yy = y-1; break;
		 	case 7 :
		 		xx=x; yy=y-1; break;
		 	case 8 :
		 		xx=1+x; yy=y-1; break;
			}
		  if(xx>=0 && xx<parent.ncols && yy>=0 && yy<parent.nrows && ras[yy*(parent.ncols)+xx]>=127)
				return(new SwarmPoint(xx,yy,0,parent));
		  }
		 checked=-1;
		 return null;
		 }
		 
	public void migrate(short [] fromras, short [] toras) {
		// Vacate fromras and move to toras
		int location = y*(parent.ncols) + x;
		toras[location]=fromras[location];
		fromras[location]=0;
		}
		
	public void vacate(short [] fromras) {
		// Vacate fromras and move to toras
		fromras[y*(parent.ncols) + x]=0;
		}
		
		}

		 		