package staxx;

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.*;
import java.util.*;
import java.awt.*;  import javax.swing.*;

class ZGradFltr {
	SegmenterNode nd=null;
	double alpha=0.,beta=0.;
	double numSects = 1.;
	double botBoost=255;
	
	public ZGradFltr(SegmenterNode nd) {
		this.nd=nd;
		}

	public void tellWhatYouDo() {
		nd.talk("");
		nd.talk("The ZGradFltr adjusts brightnesses through a z-series of sections using a");
		nd.talk("linear boost/cut function of depth, z, into the stack. The value of this");
		nd.talk("quadratic function is pinned down at 33 depths in the stack: at the top and");
		nd.talk("and bottom of the stack, and at some specified interior depth.");
		nd.talk("ZGradFltr has a single input raster (which should be one of the biorad stacks).");
		nd.talk("It has four parameters:");
		nd.talk("botBoost = the boost factor at section zero;");
		nd.talk("topBoost = the boost factor at the last section.");
		nd.talk("midDepth = the section number of that interior section at which the boost factor is...");
		nd.talk("midBoost = the boost factor at section number midDepth.");
		nd.talk("");
		nd.talk("Use boost factors less than 1.0 to attenuate brightness, and boost factors");
		nd.talk("greater than 1.0 to increase brightness. This filter is intended to compensate");
		nd.talk("for loss of brightness due to bleaching and to light scattering at deeper sections.");
		}
  private double booster(double botBoost, double topBoost, int midDepth, double midBoost, int nsects, int sectNumber) {
		double boostFac;
		if(sectNumber<= midDepth ) {
			if(midDepth<=0) return((topBoost+midBoost)/2.0);
			else return(topBoost +sectNumber*(midBoost-topBoost)/(midDepth));
			}
		else {
			if(midDepth>=numSects) return(.5*(botBoost+midBoost));
			else return(midBoost + (sectNumber-midDepth)*(botBoost-midBoost)/(nsects-1-midDepth));
			}
		
  		}		
		
 public boolean doit(short [] inras, int incols, int inrows, double botBoost, double topBoost,
			int midDepth, double midBoost, int nsects, int sectnumber,boolean ifXYsect,short [] outras) {
		if(nd.diags) nd.talk("ZGradFltr ");

		if(incols != nd.rasCols || inrows != nd.rasRows) return false;
		int i, npix = incols*inrows,j,rowoff;
		short boostPix,maxglow=(short)255;
		numSects = nsects-1;
		if(numSects<1) numSects=1;
		// the "top" is section 0
		double boostFac = booster(botBoost, topBoost, midDepth, midBoost, nsects, sectnumber);
		if(ifXYsect) { // in XY section, all pixels get same boost
			nd.talk("boost all pixels by "+boostFac+", with botBoost="+botBoost+" and topBoost="+
					topBoost+" at sect "+sectnumber+" of "+nsects+"-section stack");
			for(i=0;i<npix;i++) {
			if((boostPix=(short)(boostFac*inras[i]))>maxglow)
				boostPix=maxglow;
				outras[i] = boostPix<0? 0 :boostPix;
				}
			}
		else { // it's an XZ or an ZY section so each line gets a different boost
			for(i = 0;i<inrows;i++) {
				boostFac = booster(botBoost, topBoost, midDepth, midBoost, nsects, i);
				rowoff = i*incols;
				for(j=0;j<incols;j++) {
					if((boostPix=(short)(boostFac*inras[rowoff+j]))>maxglow)
						boostPix=maxglow;
					outras[rowoff+j] = boostPix<0? 0 : boostPix;
				}
				}					
			}
				
		return true;	
		}
}