package staxx;

/* Info ...  author, copyright, bundle-type, etc */

/*  <stax program for inspecting and processing stacks of confocal micrographs from Biorad *.pic format files>
    Copyright (C) <2008>  <Garrett Odell>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;  
import javax.swing.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;

class showDeconCanvas extends Canvas   implements java.awt.event.MouseListener, java.awt.event.MouseMotionListener {	
	
	showDeconJFrame parent=null;
	int	mouseStartX, mouseStartY; 		
	int [] theIntPic=null;
	int [] histogram1=new int[256];
	int [] histogram2=new int[256];
	boolean ifPaintHistogram = false;
	double histogram1ScaleFac = .1,histogram2ScaleFac = .1;
	
	int width=0,height=0,npix;
	Image deconvolvedImage=null;

	public void paintHistograms(Graphics g) {
		// find normalization values.
		int y,x,yleft;
		g.setColor(Color.red);
		y = (int)(histogram1ScaleFac*histogram1[0]);
			if((y = height-y) < 0)
				y=0;;

		for(x=1;x<256;x++) {
			yleft=y;
			y = (int)(histogram1ScaleFac*histogram1[x]);
			if((y = height-y) < 0)
				y=0;
			g.drawLine(x-1,yleft,x,y);
			}
			
		g.setColor(Color.green);
		y = (int)(histogram2ScaleFac*histogram2[0]);
			if((y = height-y) < 0)
				y=0;;

		for(x=1;x<256;x++) {
			yleft=y;
			y = (int)(histogram2ScaleFac*histogram2[x]);
			if((y = height-y) < 0)
				y=0;
			g.drawLine(x-1,yleft,x,y);
			}
		}

	private double  getHistogramScaleFac(int [] hgram, int scaledPeakHeight) {
		int i,maxval=1,len = hgram.length, tooDim=0, dimcount = npix/4 ;
		while(dimcount>0 && tooDim<255) {
			dimcount -=  hgram[tooDim];
			++tooDim;
			}
		++tooDim;
		for(i=tooDim;i<len;i++)
			if(hgram[i]>maxval) 
				maxval = hgram[i];
		return ((double )scaledPeakHeight)/((double)maxval);
		}
			
	public void sepuku() {
		theIntPic=null;
		deconvolvedImage=null;
		parent=null;
		//    talk("        BestSegCanvas.sepuku()");
		setVisible(false);
		}

	public showDeconCanvas(int ncols,  int nrows, showDeconJFrame parent) {		this.height=nrows;
		this.width=ncols;
		this.parent=parent;
		npix = nrows*ncols;
		setBackground(Color.white);
		setVisible(true);
	   	addMouseListener(this);
	   	addMouseMotionListener(this);
		}
				
	public int newPic( byte [] theDeconRaster, int deconSectionColor,
				byte [] origRas, int origSectColor) {
		int i,pixo,pixd;
		int saturatedPixels=0;
		int alpha =(255 << 24),red,green,blue;
		if(theIntPic == null)
			theIntPic = new int[npix];
		for(i=0;i<256;i++) {
			histogram1[i]=0;
			histogram2[i]=0;
			}
		
		for(i=0;i<npix;i++) {
			red=0; green=0; blue=0;
			if(origSectColor>=0) { // negative value means original section is invisible
			  if((pixo=origRas[i]) <0) 
				pixo += 256;
			++histogram1[pixo];
			switch (origSectColor) {
				case 0: // red
						red = pixo;
					break;
				case 1: // green
					green = pixo;
					break;
				case 2: // blue
					blue = pixo;
					break;
				case 3: // gray
					red = pixo;
					blue = pixo;
					green = pixo;
					break;
				}}
				
			if((pixd=theDeconRaster[i]) <0) 
				pixd += 256;
			++histogram2[pixd];
			if(pixd == 255) ++saturatedPixels;
			switch (deconSectionColor) { // the deconvolved section overwrites the original
				case 0: // red
					red = pixd;
					break;
				case 1: // green
					green = pixd;
					break;
				case 2: // blue
					blue = pixd;
					break;
				case 3: // gray
					red = pixd;
					blue = pixd;
					green = pixd;
					break;
				}
			theIntPic[i] =  (alpha | (red << 16) | (green << 8) | (blue));
			}
		setSize(width,height);
		histogram1ScaleFac = getHistogramScaleFac(histogram1, height-50);		
		histogram2ScaleFac = getHistogramScaleFac(histogram2, height-50);
		if(histogram1ScaleFac < histogram2ScaleFac)
			histogram1ScaleFac = histogram2ScaleFac;
		else				
			histogram2ScaleFac = histogram1ScaleFac; // use the bigger one for both
	    deconvolvedImage = createImage(new MemoryImageSource(width, height,
			 	ColorModel.getRGBdefault(),theIntPic, 0, width));
		setVisible(true);
		repaint();
		return(saturatedPixels);
		}
			
	public void paint(Graphics g) {
		//talk("This should never happen. showDeconCanvas.paint() called");
		paint2D((Graphics2D)g);
		}
		
	public void update(Graphics g) {
		// talk("showDeconCanvas.update() called. Should never happen");
		paint2D((Graphics2D)g);
		}

	public void update2D(Graphics2D g) {
		paint2D(g);
		}


	public void paint2D(Graphics2D g) {
		if(theIntPic==null) return;
	    if(deconvolvedImage==null) {
	    	deconvolvedImage = createImage(new MemoryImageSource(width, height,
			 	ColorModel.getRGBdefault(), theIntPic, 0, width));
			}
		try {
			 g.drawImage(deconvolvedImage, 0, 0, width,height, this);
			 }
		  catch (Exception e) {
			System.out.println("Error painting bestPic panel: " + e.toString());
			}
		if(ifPaintHistogram)
			paintHistograms(g);			
		}

	static void talk(String stg) { // write a line on stdout
		System.out.println(stg);
		}

	public void update(Graphics2D g) {
		paint(g);
		}
	
	private int [] getGlow(int x, int y) {
		int [] glo = {0,0,0,0};
		if(x<0 || x>=width || y<0 || y>= height)
			return glo;
		int offset = y*width +x;
		int pix = theIntPic[offset];
		glo[0] = (pix & (255<<16)) >> 16;
		glo[1] = (pix & (255<<8)) >> 8;
		glo[2] = pix & 255;
		if((glo[3]=glo[0]) < glo[1])
			glo[3]=glo[1];
		if(glo[3] < glo[2])
			glo[3]=glo[2]; // the return array has {red,green,blue,max}
		return glo;
		}
		
	private int [] getNbhdGlow(int x, int y) {
		int [] nbhdGlow = {0,0,0,0};
		if(x<0 || x>=width || y<0 || y>= height)
			return nbhdGlow;
		int yi,xj,i,j,k,npx=0;
		int [] glo;
		for(i=-2; i<3; i++)
			if((yi=y+i) >= 0 && yi < width)
				for(j=-2;j<3;j++)
					if((xj=x+j) >= 0 && xj<width) {
						glo = getGlow(xj,yi);
						++npx;
						for(k=0;k<4;k++)
							nbhdGlow[k] += glo[k];
						}
		
		for(k=0;k<4;k++)
			nbhdGlow[k] = nbhdGlow[k]/npx;;
		return nbhdGlow;
		}
		
	public void mouseMoved(java.awt.event.MouseEvent event){
		int x = event.getX();
		int y = event.getY();
		int [] bri = getGlow(x,y);
		int [] avg = getNbhdGlow(x,y);
		parent.pixelReport(x,y,bri,avg);
		}
			
	public void mouseDragged(java.awt.event.MouseEvent event){
		int x=event.getX(),y=event.getY();
		}
	
// catch all other mouse events 
	
	public void mousePressed(java.awt.event.MouseEvent event) {
		}
				
	public void mouseReleased(java.awt.event.MouseEvent event) {
		int x= event.getX(), y=event.getY();
		}
		
		
	public void mouseClicked(java.awt.event.MouseEvent event) {
		int x=event.getX(),y=event.getY();
		}
		
	public void mouseEntered(java.awt.event.MouseEvent event) {
		// talk("Mouse entered at ("+event.getX()+", "+event.getY()+")");
		}
		
	public void mouseExited(java.awt.event.MouseEvent event) {
		// talk("Mouse exited at ("+event.getX()+", "+event.getY()+")");
		}

}